/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.featuretree;

import java.util.List;
import java.util.logging.ErrorManager;

import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import eu.etaxonomy.cdm.api.service.DeleteResult;
import eu.etaxonomy.cdm.api.service.IFeatureTreeService;
import eu.etaxonomy.cdm.model.description.FeatureTree;
import eu.etaxonomy.taxeditor.model.DeleteResultMessagingUtils;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * <p>SelectFeatureTreeWizardPage class.</p>
 *
 * @author n.hoffmann
 * @created Aug 5, 2010
 * @version 1.0
 */
public class SelectFeatureTreeWizardPage extends WizardPage implements ISelectionChangedListener, ModifyListener{

	private ListViewer viewer;
	private Text text_title;
	private Button button_add;
	private Button button_remove;

	/**
	 * <p>Constructor for SelectFeatureTreeWizardPage.</p>
	 *
	 * @param pageName a {@link java.lang.String} object.
	 */
	protected SelectFeatureTreeWizardPage(String pageName) {
		super(pageName);
		setMessage("Select a Feature Tree or create a new one.");
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
	 */
	/** {@inheritDoc} */
	@Override
	public void createControl(Composite parent) {
		Composite composite = new Composite(parent, SWT.NULL);

		composite.setLayout(new GridLayout());

		Composite composite_treeContent = new Composite(composite, SWT.NULL);
		composite_treeContent.setLayoutData(new GridData(SWT.FILL, SWT.FILL,
				true, true));
		composite_treeContent.setLayout(new GridLayout(2, false));

		viewer = new ListViewer(composite_treeContent);
		viewer.getControl().setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));

		button_remove = new Button(composite_treeContent, SWT.PUSH);
		button_remove.setText("Remove");
		button_remove.setLayoutData(new GridData(SWT.RIGHT, SWT.TOP, false, false));

		Composite composite_treeTitle = new Composite(composite, SWT.NULL);
		composite_treeTitle.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true,
				false));
		composite_treeTitle.setLayout(new GridLayout(3, false));



		Label label_title = new Label(composite_treeTitle, SWT.NULL);
		label_title.setText("New Feature Tree");

		text_title = new Text(composite_treeTitle, SWT.NULL);
		text_title.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

		button_add = new Button(composite_treeTitle, SWT.PUSH);
		button_add.setText("Add");

		viewer.setContentProvider(new FeatureTreeContentProvider());
		viewer.setLabelProvider(new FeatureTreeLabelProvider());

		viewer.addSelectionChangedListener(this);

		text_title.addModifyListener(this);
		button_add.addSelectionListener(new AddButtonSelectionListener());
		button_remove.addSelectionListener(new RemoveButtonSelectionListener());

		List<FeatureTree> input = CdmStore.getService(IFeatureTreeService.class).list(FeatureTree.class, null, null, null, null);

		viewer.setInput(input);
		modifyText(null);
		setControl(composite);
	}

	/** {@inheritDoc} */
	@Override
	public void selectionChanged(SelectionChangedEvent event) {
		IStructuredSelection selection = (IStructuredSelection) event.getSelection();

		if(selection.size() == 1){
			FeatureTree selectedFeatureTree = (FeatureTree) selection.getFirstElement();
			((FeatureTreeEditorWizard) getWizard()).setSelectedFeatureTree(selectedFeatureTree);

		}

		setPageComplete(true);

		button_remove.setEnabled(selection.size() >= 1);
	}

	/** {@inheritDoc} */
	@Override
	public boolean canFlipToNextPage() {
		return ((IStructuredSelection) viewer.getSelection()).size() == 1;
	}

	/** {@inheritDoc} */
	@Override
	public void modifyText(ModifyEvent e) {
		button_add.setEnabled(! text_title.getText().isEmpty());
	}

	private class AddButtonSelectionListener extends SelectionAdapter {
		@Override
        public void widgetSelected(SelectionEvent e) {
			FeatureTree featureTree = FeatureTree.NewInstance();
			featureTree.setTitleCache(text_title.getText(), true);

			viewer.add(featureTree);
			CdmStore.getService(IFeatureTreeService.class).merge(featureTree, true);
			
			text_title.setText("");
			viewer.setSelection(new StructuredSelection(featureTree));

		}
	}

	private class RemoveButtonSelectionListener extends SelectionAdapter {
		@Override
        public void widgetSelected(SelectionEvent e) {
			IStructuredSelection selection = (IStructuredSelection) viewer.getSelection();
			Object source = e.getSource();
			for(Object element : selection.toArray()){
				viewer.remove(element);
				((FeatureTreeEditorWizard) getWizard()).setSelectedFeatureTree(null);
				DeleteResult result =	CdmStore.getService(IFeatureTreeService.class).delete(((FeatureTree) element).getUuid());

				if (result.isError()){
					DeleteResultMessagingUtils.messageDialogWithDetails(result, "The delete of the feature tree was not successful.", null);
				}
				viewer.setSelection(new StructuredSelection(new Object[0]));
			}
		}
	}
}
