/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.preference;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import java.util.StringTokenizer;
import java.util.UUID;

import org.apache.commons.lang.StringUtils;
import org.eclipse.equinox.internal.p2.ui.model.MetadataRepositoryElement;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;

import eu.etaxonomy.cdm.api.application.ICdmRepository;
import eu.etaxonomy.cdm.api.facade.DerivedUnitFacadeConfigurator;
import eu.etaxonomy.cdm.api.service.IFeatureTreeService;
import eu.etaxonomy.cdm.api.service.ITermService;
import eu.etaxonomy.cdm.api.service.config.FindTaxaAndNamesConfiguratorImpl;
import eu.etaxonomy.cdm.api.service.config.IFindTaxaAndNamesConfigurator;
import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.common.ICdmBase;
import eu.etaxonomy.cdm.model.common.IDefinedTerm;
import eu.etaxonomy.cdm.model.common.ISimpleTerm;
import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.cdm.model.common.MarkerType;
import eu.etaxonomy.cdm.model.common.TermBase;
import eu.etaxonomy.cdm.model.description.FeatureTree;
import eu.etaxonomy.cdm.model.metadata.CdmPreference;
import eu.etaxonomy.cdm.model.metadata.CdmPreference.PrefKey;
import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.cdm.model.metadata.PreferenceSubject;
import eu.etaxonomy.cdm.model.name.NomenclaturalCode;
import eu.etaxonomy.cdm.strategy.match.DefaultMatchStrategy;
import eu.etaxonomy.cdm.strategy.match.IMatchStrategy;
import eu.etaxonomy.cdm.strategy.match.MatchException;
import eu.etaxonomy.cdm.strategy.match.MatchMode;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.model.NomenclaturalCodeHelper;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.internal.TaxeditorStorePlugin;
import eu.etaxonomy.taxeditor.ui.dialog.DefaultLanguageDialog;

/**
 * <p>
 * PreferencesUtil class.
 * </p>
 *
 * @author p.ciardelli
 * @author n.hoffmann
 * @created 05.12.2008
 * @version 1.0
 */
public class PreferencesUtil implements IPreferenceKeys {

	/**
	 *
	 */
	public static final String PREFERRED_TERMS_CHANGE = "preferred_terms";

	public static final String P2_REPOSITORIES_DELIM = ",";
	public static final String P2_REPOSITORY_FIELDS_DELIM = ";";



	/**
	 * <p>
	 * getPreferenceStore
	 * </p>
	 *
	 * @return a {@link org.eclipse.jface.preference.IPreferenceStore} object.
	 */
	public static IPreferenceStore getPreferenceStore() {
		return TaxeditorStorePlugin.getDefault().getPreferenceStore();
	}

	/**
	 * <p>
	 * setPreferredNomenclaturalCode
	 * </p>
	 *
	 * @param preferredCode
	 *            a {@link eu.etaxonomy.cdm.model.name.NomenclaturalCode}
	 *            object.
	 */
	public static CdmPreference setPreferredNomenclaturalCode(
			NomenclaturalCode preferredCode, boolean local) {
	    if (local){
	        getPreferenceStore().setValue(PREFERRED_NOMENCLATURAL_CODE_KEY,
                    getPreferenceKey(preferredCode));
	    }
	    else{
    		ICdmRepository controller;
    		controller = CdmStore.getCurrentApplicationConfiguration();
    		PrefKey key = CdmPreference.NewKey(PreferenceSubject.NewDatabaseInstance(), PreferencePredicate.NomenclaturalCode);
    		CdmPreference preference = null;
    		if (controller == null){
    			return null;
    		}
    		if (preferredCode == null){
    			preference = controller.getPreferenceService().find(key);
    			if (preference == null){
    				return null;
    			} else{
    				int index = StringUtils.lastIndexOf(preference.getValue(), ".");
    				UUID uuid = UUID.fromString(preference.getValue().substring(index +1, preference.getValue().length()));
    				preferredCode = NomenclaturalCode.getByUuid(uuid);

    				getPreferenceStore().setValue(CDM_NOMENCLATURAL_CODE_KEY,
    	                    getPreferenceKey(preferredCode));
    				getPreferenceStore().setValue(ALLOW_OVERRIDE_NOMENCLATURAL_CODE_KEY,preference.isAllowOverride());
    				return preference;
    			}
    		} else{
    			preference = CdmPreference.NewInstance(PreferenceSubject.NewDatabaseInstance(), PreferencePredicate.NomenclaturalCode, preferredCode.getKey());
    			controller.getPreferenceService().set(preference);

    		}
	    }
        return null;



	}

	public static NomenclaturalCode getPreferredNomenclaturalCode(){
	    if (getPreferenceStore().getBoolean(OVERRIDE_NOMENCLATURAL_CODE_KEY)) {
            return getPreferredNomenclaturalCode(true);
        } else{
            return getPreferredNomenclaturalCode(false);
        }
	}

	/**
	 * <p>
	 * getPreferredNomenclaturalCode
	 * </p>
	 *
	 * @return a {@link eu.etaxonomy.cdm.model.name.NomenclaturalCode} object.
	 */
	public static NomenclaturalCode getPreferredNomenclaturalCode(boolean useLocal) {

		CdmPreference pref = null;
		if (!useLocal){
		    pref = getPreferenceFromDB(PreferencePredicate.NomenclaturalCode);
		}

	    String preferredCode;
	    if(pref == null){
	    	preferredCode = getPreferenceStore().getString(
					PREFERRED_NOMENCLATURAL_CODE_KEY);

	    }else{
	    	preferredCode = pref.getValue();
	    }

		for (NomenclaturalCode code : NomenclaturalCodeHelper.getAllCodes()) {
			if (getPreferenceKey(code).equals(preferredCode)) {
				return code;
			}
		}
		return null;
	}


    private static CdmPreference getPreferenceFromDB(PreferencePredicate predicate){
        ICdmRepository controller;
        CdmPreference pref = null;

        try{
            controller = CdmStore.getCurrentApplicationConfiguration();
            PrefKey key = CdmPreference.NewKey(PreferenceSubject.NewDatabaseInstance(), predicate);
            pref = controller.getPreferenceService().find(key);
        }catch(Exception e){
            e.printStackTrace();
        }

        return pref;

    }


	public static String getPreferredDefaultLangugae(){
	    String preferredLanguage = getPreferenceStore().getString(DEFAULT_LANGUAGE_EDITOR);
	    if(StringUtils.isNotEmpty(preferredLanguage) && StringUtils.isNotBlank(preferredLanguage)){
	        return preferredLanguage;
	    }
	    return null;
	}

	/**
	 * Get the match strategy for the given class that was stored in preferences
	 * or the default strategy if it was not stored in preferences
	 *
	 * @param clazz
	 *            a {@link java.lang.Class} object.
	 * @return a {@link eu.etaxonomy.cdm.strategy.match.IMatchStrategy} object.
	 */
	public static IMatchStrategy getMatchStrategy(Class clazz) {
		String className = clazz.getName();
		if (getPreferenceStore().getBoolean(MATCH_STRATEGY_PREFIX + className)) {
			IMatchStrategy matchStrategy = getDefaultMatchStrategy(clazz);

			for (String fieldName : matchStrategy.getMatchFieldPropertyNames()) {
				String matchModeName = getPreferenceStore().getString(
						getMatchStrategyFieldName(className, fieldName));
				MatchMode matchMode = MatchMode.valueOf(matchModeName);
				try {
					matchStrategy.setMatchMode(fieldName, matchMode);
				} catch (MatchException e) {
					MessagingUtils.error(PreferencesUtil.class, e);
					throw new RuntimeException(e);
				}
			}

			return matchStrategy;
		}
		return getDefaultMatchStrategy(clazz);
	}

	/**
	 * Stores a matchStrategy into the preference store.
	 *
	 * @param matchStrategy
	 *            a {@link eu.etaxonomy.cdm.strategy.match.IMatchStrategy}
	 *            object.
	 */
	public static void setMatchStrategy(IMatchStrategy matchStrategy) {
		String className = matchStrategy.getMatchClass().getName();
		getPreferenceStore().setValue(MATCH_STRATEGY_PREFIX + className, true);

		Set<String> matchFields = matchStrategy.getMatchFieldPropertyNames();

		for (String fieldName : matchFields) {
			getPreferenceStore().setValue(
					getMatchStrategyFieldName(className, fieldName),
					matchStrategy.getMatchMode(fieldName).name());
		}
	}

	/**
	 * Helper method to create the preference property for a match field.
	 *
	 * @param className
	 * @param fieldName
	 * @return
	 */
	private static String getMatchStrategyFieldName(String className,
			String fieldName) {
		return MATCH_STRATEGY_PREFIX + className + "." + fieldName;
	}

	/**
	 * Returns the default match strategy for a given class.
	 *
	 * @param clazz
	 *            a {@link java.lang.Class} object.
	 * @return a {@link eu.etaxonomy.cdm.strategy.match.IMatchStrategy} object.
	 */
	public static IMatchStrategy getDefaultMatchStrategy(Class clazz) {
		return DefaultMatchStrategy.NewInstance(clazz);
	}

	/**
	 * <p>
	 * getDateFormatPattern
	 * </p>
	 *
	 * @return a {@link java.lang.String} object.
	 */
	public static String getDateFormatPattern() {
		// TODO make this configurable in properties
		String pattern = "Y-M-d H:m";
		return pattern;
	}

	/**
	 * <p>
	 * addTermToPreferredTerms
	 * </p>
	 *
	 * @param term
	 *            a T object.
	 * @param <T>
	 *            a T object.
	 */
	public static <T extends TermBase> void addTermToPreferredTerms(T term) {

		// VocabularyEnum vocabulary =
		// VocabularyEnum.getVocabularyEnum(term.getClass());
		//
		// getPreferenceStore().setValue(getPreferenceKey(term),
		// VocabularyStore.getTermVocabulary(vocabulary).getTerms().contains(term));
		//
		// firePreferencesChanged(term.getClass());
	}

	/**
	 * Construct a unique key using the CdmBase object's uuid
	 *
	 * @param cdmBase
	 * @return
	 */
	private static String getPreferenceKey(ICdmBase cdmBase) {
		cdmBase = HibernateProxyHelper.deproxy(cdmBase);

		String key = cdmBase.getClass().getName().concat(".")
				.concat(cdmBase.getUuid().toString());
		if (key.contains("javassist")) {
			MessagingUtils.info("proxy");
		}
		return key;
	}

	/**
	 * Construct a unique key using the CdmBase object's uuid
	 *
	 * @param cdmBase
	 * @return
	 */
	public static String getPreferenceKey(ISimpleTerm simpleTerm) {
		simpleTerm = HibernateProxyHelper.deproxy(simpleTerm);
		String key = simpleTerm.getClass().getName().concat(".")
				.concat(simpleTerm.getUuid().toString());
		if (key.contains("javassist")) {
			MessagingUtils.warn(PreferencesUtil.class,
					"Trying to persist a preference based on a proxy class.");
		}
		return key;
	}



	/**
	 * Construct a unique key using the CdmBase object's uuid
	 *
	 * @param cdmBase
	 * @return
	 */
	public static String getPreferenceKey(IDefinedTerm definedTerm) {
		definedTerm = HibernateProxyHelper.deproxy(definedTerm);
		String key = definedTerm.getClass().getName().concat(".")
				.concat(definedTerm.getUuid().toString());
		if (key.contains("javassist")) {
			MessagingUtils.warn(PreferencesUtil.class,
					"Trying to persist a preference based on a proxy class.");
		}
		return key;
	}

	/**
	 * Retrieves search preferences from the preference store
	 *
	 * @return an {@link ITaxonServiceConfigurator} to pass to search methods
	 */
	public static IFindTaxaAndNamesConfigurator getSearchConfigurator() {
		IFindTaxaAndNamesConfigurator configurator = initializeSearchConfigurator();

		configurator.setDoTaxa(getPreferenceStore().getBoolean(
				TAXON_SERVICE_CONFIGURATOR_TAXA));
		configurator.setDoSynonyms(getPreferenceStore().getBoolean(
				TAXON_SERVICE_CONFIGURATOR_SYNONYMS));
		configurator.setDoNamesWithoutTaxa(getPreferenceStore().getBoolean(
				TAXON_SERVICE_CONFIGURATOR_NAMES));
		configurator.setDoTaxaByCommonNames(getPreferenceStore().getBoolean(
				TAXON_SERVICE_CONFIGURATOR_COMMON_NAMES));
		//configurator.setMatchMode(eu.etaxonomy.cdm.persistence.query.MatchMode.valueOf(getPreferenceStore().getString(TAXON_SERVICE_CONFIGURATOR_MATCH_MODE)));

		return configurator;
	}

	/**
	 * create new preferences, setting all search options to true
	 *
	 * @return a
	 *         {@link eu.etaxonomy.cdm.api.service.config.ITaxonServiceConfigurator}
	 *         object.
	 */
	public static IFindTaxaAndNamesConfigurator initializeSearchConfigurator() {
		IFindTaxaAndNamesConfigurator configurator = new FindTaxaAndNamesConfiguratorImpl();

		configurator.setDoTaxa(true);
		configurator.setDoSynonyms(true);
		configurator.setDoNamesWithoutTaxa(true);
		configurator.setDoTaxaByCommonNames(true);

		configurator.setTaxonPropertyPath(Arrays.asList("$", "titleCache",
				"name", "name.$", "relationsFromThisTaxon.$"));

		configurator.setSynonymPropertyPath(Arrays.asList("$", "titleCache",
				"name", "name.$", "synonymRelations.relatedTo.*"));

		// DEFAULT VALUES
		// match mode is a simple like, actually all other match modes are kind
		// of bogus
		configurator
				.setMatchMode(eu.etaxonomy.cdm.persistence.query.MatchMode.ANYWHERE);
		// we set page number and size here as this should always be unlimited
		configurator.setPageNumber(0);
		// TODO currently limit results to 10000
		configurator.setPageSize(10000);

		return configurator;
	}

	/**
	 * Store search preferences
	 *
	 * @param configurator
	 *            a
	 *            {@link eu.etaxonomy.cdm.api.service.config.ITaxonServiceConfigurator}
	 *            object.
	 */
	public static void setSearchConfigurator(
			IFindTaxaAndNamesConfigurator configurator) {
		getPreferenceStore().setValue(TAXON_SERVICE_CONFIGURATOR_TAXA,
				configurator.isDoTaxa());
		getPreferenceStore().setValue(TAXON_SERVICE_CONFIGURATOR_SYNONYMS,
				configurator.isDoSynonyms());
		getPreferenceStore().setValue(TAXON_SERVICE_CONFIGURATOR_NAMES,
				configurator.isDoNamesWithoutTaxa());
		getPreferenceStore().setValue(TAXON_SERVICE_CONFIGURATOR_COMMON_NAMES,
				configurator.isDoTaxaByCommonNames());
	}

	/**
	 * <p>
	 * firePreferencesChanged
	 * </p>
	 *
	 * @param clazz
	 *            a {@link java.lang.Class} object.
	 */
	public static void firePreferencesChanged(Class clazz) {
		getPreferenceStore().firePropertyChangeEvent(PREFERRED_TERMS_CHANGE,
				null, clazz);
	}

	/**
	 * Set default values for preferences
	 */
	public static void setDefaults() {
		getPreferenceStore().setDefault(TAXON_SERVICE_CONFIGURATOR_TAXA, true);
		getPreferenceStore().setDefault(TAXON_SERVICE_CONFIGURATOR_SYNONYMS,
				true);
		getPreferenceStore().setDefault(EDIT_MAP_SERVICE_ACCES_POINT,
				"http://edit.africamuseum.be/edit_wp5/v1.2/rest_gen.php");
		//FIXME : changed default for SHOULD_CONNECT_AT_STARTUP to false (ticket 3828) until resolution
		getPreferenceStore().setDefault(SHOULD_CONNECT_AT_STARTUP, false);
		getPreferenceStore().setDefault(OPENURL_ACCESS_POINT,
				"http://www.biodiversitylibrary.org/openurl");
		getPreferenceStore().setDefault(OPENURL_IMAGE_MAX_WIDTH, "1000");
		getPreferenceStore().setDefault(OPENURL_IMAGE_MAX_HEIGHT, "1000");
		getPreferenceStore().setDefault(IPreferenceKeys.DISTRIBUTION_AREA_PREFRENCES_ACTIVE, false);
		getPreferenceStore().setDefault(CHECKLIST_ID_IN_VOCABULARY, true);
		getPreferenceStore().setDefault(SHOW_NAME_DETAILS_SECTION_ATOMISED_EPITHETS, true);
		getPreferenceStore().setDefault(SHOW_NAME_DETAILS_SECTION_AUTHORSHIP_CACHE, true);
		getPreferenceStore().setDefault(SHOW_NAME_DETAILS_SECTION_AUTHORSHIP, true);
		getPreferenceStore().setDefault(SHOW_NAME_DETAILS_SECTION_HYBRID, true);
		getPreferenceStore().setDefault(SHOW_NAME_DETAILS_SECTION_LSID, true);
		getPreferenceStore().setDefault(SHOW_NAME_DETAILS_SECTION_NAME_RELATIONSHIP, true);
		getPreferenceStore().setDefault(SHOW_NAME_DETAILS_SECTION_NAMECACHE, true);
		getPreferenceStore().setDefault(SHOW_NAME_DETAILS_SECTION_NOMENCLATURAL_CODE, true);
		getPreferenceStore().setDefault(SHOW_NAME_DETAILS_SECTION_NOMENCLATURAL_REFERENCE, true);
		getPreferenceStore().setDefault(SHOW_NAME_DETAILS_SECTION_NOMENCLATURAL_STATUS, true);
		getPreferenceStore().setDefault(SHOW_NAME_DETAILS_SECTION_PROTOLOGUE, true);
		getPreferenceStore().setDefault(SHOW_NAME_DETAILS_SECTION_RANK, true);
		getPreferenceStore().setDefault(SHOW_NAME_DETAILS_SECTION_TYPE_DESIGNATION, true);
		getPreferenceStore().setDefault(SORT_NODES_NATURALLY, false);
		getPreferenceStore().setDefault(SORT_NODES_ALPHABETICALLY, false);

	}

	/**
	 * <p>
	 * checkNomenclaturalCode
	 * </p>
	 */
	public static void checkNomenclaturalCode() {
		// First time Editor is opened, no nomenclatural code has been set


		if (PreferencesUtil.getPreferredNomenclaturalCode(true) == null) {
			PreferencesUtil.setPreferredNomenclaturalCode(NomenclaturalCode.ICNAFP, true);
		}



	}
	public static void setNomenclaturalCodePreferences(){
	    ICdmRepository controller;
        controller = CdmStore.getCurrentApplicationConfiguration();
        PrefKey key = CdmPreference.NewKey(PreferenceSubject.NewDatabaseInstance(), PreferencePredicate.NomenclaturalCode);
        CdmPreference preference = null;
        if (controller == null){
            return ;
        }
        preference = controller.getPreferenceService().find(key);
        if (preference == null){
            return;
        }
        getPreferenceStore().setValue(ALLOW_OVERRIDE_NOMENCLATURAL_CODE_KEY, preference.isAllowOverride());

        int index = StringUtils.lastIndexOf(preference.getValue(), ".");
        UUID uuid = UUID.fromString(preference.getValue().substring(index +1, preference.getValue().length()));
        NomenclaturalCode preferredCode = NomenclaturalCode.getByUuid(uuid);

        getPreferenceStore().setValue(CDM_NOMENCLATURAL_CODE_KEY,
                getPreferenceKey(preferredCode));

        if (!preference.isAllowOverride()){
            getPreferenceStore().setValue(PREFERRED_NOMENCLATURAL_CODE_KEY,
                    getPreferenceKey(preferredCode));
        }
	}

	public static void checkDefaultLanguage(){
	    if(PreferencesUtil.getPreferredDefaultLangugae() == null){
	       Shell shell = AbstractUtility.getShell();
	       int open = new DefaultLanguageDialog(shell).open();
	       if(open == Window.OK){
	           PlatformUI.getWorkbench().restart();
	       }
	    }else{
	        //TODO:In case of a reinstall, the config.ini will be overwritten
	        //     here you create config.ini with the stored key from preferences
	    }
	}

	/**
	 * <p>
	 * getMapServiceAccessPoint
	 * </p>
	 *
	 * @return a {@link java.lang.String} object.
	 */
	public static String getMapServiceAccessPoint() {
		return getPreferenceStore().getString(EDIT_MAP_SERVICE_ACCES_POINT);
	}

	/**
	 * <p>
	 * shouldConnectAtStartUp
	 * </p>
	 *
	 * @return a boolean.
	 */
	public static boolean shouldConnectAtStartUp() {
		//FIXME :  force SHOULD_CONNECT_AT_STARTUP to false (ticket 3828) until resolution
		//return getPreferenceStore().getBoolean(SHOULD_CONNECT_AT_STARTUP);
		return false;
	}

	/**
	 * <p>
	 * getDefaultFeatureTreeForTextualDescription
	 * </p>
	 *
	 * @return a {@link eu.etaxonomy.cdm.model.description.FeatureTree} object.
	 */
	public static FeatureTree getDefaultFeatureTreeForTextualDescription() {
		String uuidString = getPreferenceStore().getString(
				FEATURE_TREE_DEFAULT_TEXT);
		return CdmUtils.isEmpty(uuidString) ? null : CdmStore.getService(
				IFeatureTreeService.class).load(UUID.fromString(uuidString));
	}

	/**
	 * <p>
	 * getDefaultFeatureTreeForStructuredDescription
	 * </p>
	 *
	 * @return a {@link eu.etaxonomy.cdm.model.description.FeatureTree} object.
	 */
	public static FeatureTree getDefaultFeatureTreeForStructuredDescription() {
		String uuidString = getPreferenceStore().getString(
				FEATURE_TREE_DEFAULT_STRUCTURE);
		return CdmUtils.isEmpty(uuidString) ? null : CdmStore.getService(
				IFeatureTreeService.class).load(UUID.fromString(uuidString));
	}

	/**
	 * <p>
	 * setSortRanksHierarchichally
	 * </p>
	 *
	 * @param selection
	 *            a boolean.
	 */
	public static void setSortRanksHierarchichally(boolean selection) {
		getPreferenceStore().setValue(SORT_RANKS_HIERARCHICHALLY, selection);
	}

	/**
	 * <p>
	 * getSortRanksHierarchichally
	 * </p>
	 *
	 * @return a boolean.
	 */
	public static boolean getSortRanksHierarchichally() {
		return getPreferenceStore().getBoolean(SORT_RANKS_HIERARCHICHALLY);
	}

	public static boolean isMultilanguageTextEditingCapability() {
		return getPreferenceStore().getBoolean(
				MULTILANGUAGE_TEXT_EDITING_CAPABILITY);
	}

	public static Language getGlobalLanguage() {


		String languageUuidString = getPreferenceStore().getString(
				GLOBAL_LANGUAGE_UUID);

		if(!CdmStore.isActive()) {
            MessagingUtils.noDataSourceWarningDialog(languageUuidString);
            return null;
        }

		if (CdmUtils.isBlank(languageUuidString)) {
			return Language.getDefaultLanguage();
		}

		UUID languageUuid = UUID.fromString(languageUuidString);
		return (Language) CdmStore.getService(ITermService.class).load(
				languageUuid);
	}

	public static void setGlobalLanguage(Language language) {
	    if(language != null) {
	        getPreferenceStore().setValue(GLOBAL_LANGUAGE_UUID,language.getUuid().toString());
	        CdmStore.setDefaultLanguage(language);
	    }

	}

	/**
	 * @return
	 */
	public static Map<MarkerType, Boolean> getEditMarkerTypePreferences() {
		List<MarkerType> markerTypes = CdmStore.getTermManager()
				.getPreferredTerms(MarkerType.class);

		Map<MarkerType, Boolean> result = new HashMap<MarkerType, Boolean>();

		for (MarkerType markerType : markerTypes) {
			String name = getMarkerTypeEditingPreferenceKey(markerType);
			Boolean value = getPreferenceStore().getBoolean(name);

			result.put(markerType, value);
		}

		return result;
	}

	/**
	 * @param markerTypeEditingMap
	 */
	public static void setEditMarkerTypePreferences(
			Map<MarkerType, Boolean> markerTypeEditingMap) {
		for (MarkerType markerType : markerTypeEditingMap.keySet()) {
			String name = getMarkerTypeEditingPreferenceKey(markerType);
			getPreferenceStore().setValue(name,
					markerTypeEditingMap.get(markerType));
		}

	}

	private static String getMarkerTypeEditingPreferenceKey(
			MarkerType markerType) {
		markerType = HibernateProxyHelper.deproxy(markerType);
		return markerType.getClass().getName() + EDIT_MARKER_TYPE_PREFIX;
	}

	/**
	 * <p>
	 * setEditMarkerTypePreference
	 * </p>
	 *
	 * @param input
	 *            a {@link org.eclipse.ui.IEditorInput} object.
	 * @param markerType
	 *            a {@link eu.etaxonomy.cdm.model.common.MarkerType} object.
	 * @param edit
	 *            a boolean.
	 */
	public static void setEditMarkerTypePreference(MarkerType markerType,
			boolean edit) {
		getPreferenceStore().setValue(
				getMarkerTypeEditingPreferenceKey(markerType), edit);
	}

	/**
	 * @return
	 */
	public static DerivedUnitFacadeConfigurator getDerivedUnitConfigurator() {
		DerivedUnitFacadeConfigurator configurator = DerivedUnitFacadeConfigurator
				.NewInstance();
		configurator.setMoveDerivedUnitMediaToGallery(true);
		configurator.setMoveFieldObjectMediaToGallery(true);
		return configurator;
	}

	/**
	 * This method will write language properties to the config.ini located in the configuration folder
	 * of the Taxonomic Ediitor. <b>This method is only used to set the default language for Taxonomic Editor.</b>
	 *
	 * @param setLanguage 0 is for german and 1 for english.
	 * @throws IOException
	 */
    public void writePropertyToConfigFile(int setLanguage) throws IOException {
        File file = org.eclipse.core.runtime.preferences.ConfigurationScope.INSTANCE.getLocation().toFile();
        //give warning to user if the directory has no write access
        if(file == null){
            throw new IOException();
        }
        Properties properties = load(file.getAbsolutePath()+"/config.ini");
        switch(setLanguage){
        case 0:
            properties.setProperty("osgi.nl", "de");
            PreferencesUtil.getPreferenceStore().setValue(IPreferenceKeys.DEFAULT_LANGUAGE_EDITOR, "de");
            break;
        case 1:
            properties.setProperty("osgi.nl", "en");
            PreferencesUtil.getPreferenceStore().setValue(IPreferenceKeys.DEFAULT_LANGUAGE_EDITOR, "en");
            break;
        default:
            break;
        }
        save(file+"/config.ini", properties);
    }

    /**
     * This method loads a property from a given file and returns it.
     *
     * @param filename
     * @return
     * @throws IOException
     */
    private Properties load(String filename) throws IOException {
        FileInputStream in = new FileInputStream(filename);
        Properties prop = new Properties();
        prop.load(in);
        in.close();
        return prop;
    }

    /**
     * This method saves a property to the specified file.
     *
     * @param filename
     * @param properties
     * @throws IOException
     */
    private void save(String filename, Properties properties) throws IOException{
        FileOutputStream fos =  new FileOutputStream(filename);
        properties.store(fos, "");
        fos.close();
    }

    /**
     * Saves a list of P2 Metadata Repositories as string with specified delimiters
     *
     * @param p2Repos
     */
    public static void setP2Repositories(List<MetadataRepositoryElement> p2Repos) {
        StringBuilder sb = new StringBuilder();
        for(MetadataRepositoryElement p2Repo : p2Repos) {
            sb.append(P2_REPOSITORIES_DELIM);
            if(p2Repo.getName() == null || p2Repo.getName().isEmpty()) {
                sb.append("-");
            } else {
                sb.append(p2Repo.getName());
            }
            sb.append(P2_REPOSITORY_FIELDS_DELIM);
            sb.append(p2Repo.getLocation().toString());
            sb.append(P2_REPOSITORY_FIELDS_DELIM);
            sb.append(String.valueOf(p2Repo.isEnabled()));
        }
        getPreferenceStore().setValue(P2_REPOSITORY_LIST, sb.toString());
    }


    /**
     * Retrieves a list of previously saved P2 repositories
     *
     * @return
     */
    public static List<MetadataRepositoryElement> getP2Repositories() {
        List<MetadataRepositoryElement> p2Repos = new ArrayList<MetadataRepositoryElement>();
        String p2ReposPref =  getPreferenceStore().getString(P2_REPOSITORY_LIST);
        if(p2ReposPref != null && !p2ReposPref.isEmpty()) {
            StringTokenizer p2ReposPrefST = new StringTokenizer(p2ReposPref,P2_REPOSITORIES_DELIM);

            while(p2ReposPrefST.hasMoreTokens()) {
                String p2RepoStr = p2ReposPrefST.nextToken();
                StringTokenizer p2ReposStrST = new StringTokenizer(p2RepoStr,P2_REPOSITORY_FIELDS_DELIM);
                if(p2ReposStrST.countTokens()==3) {
                    String nickname = p2ReposStrST.nextToken();
                    URI uri = null;
                    try {
                        uri = new URI(p2ReposStrST.nextToken());
                    } catch (URISyntaxException e) {
                        continue;
                    }
                    boolean enabled = Boolean.parseBoolean(p2ReposStrST.nextToken());
                    MetadataRepositoryElement mre = new MetadataRepositoryElement(null, uri, true);
                    mre.setNickname(nickname);
                    mre.setEnabled(enabled);
                    p2Repos.add(mre);
                }
            }
        }

        return p2Repos;
    }

    /**
     * enables/disables nested composite. <br>
     *
     * @param ctrl - Composite to be en-/disabeld
     * @param enabled - boolean
     */
    public static void recursiveSetEnabled(Control ctrl, boolean enabled) {
        if (ctrl instanceof Composite) {
            Composite comp = (Composite) ctrl;
            for (Control c : comp.getChildren()) {
                recursiveSetEnabled(c, enabled);
            }
        } else {
            ctrl.setEnabled(enabled);
        }
    }

    /**
	 * <p>
	 * getSortRanksNaturally
	 * </p>
    	 *
    	 * @return a boolean.
	 */
	public static boolean getSortNodesNaturally() {
		return getPreferenceStore().getBoolean(SORT_NODES_NATURALLY);
	}

	/**
	 * <p>
	 * setSortRanksNaturally
	 * </p>
	 *
	 * @param selection
	 *            a boolean.
	 */
	public static void setSortNodesNaturally(boolean selection) {
		getPreferenceStore().setValue(SORT_NODES_NATURALLY, selection);
	}


	/**
	 * <p>
	 * getSortRanksNaturally
	 * </p>
	 *
	 * @return a boolean.
	 */
	public static boolean getSortNodesStrictlyAlphabetically() {
		return getPreferenceStore().getBoolean(SORT_NODES_ALPHABETICALLY);
	}

	/**
	 * <p>
	 * setSortRanksNaturally
	 * </p>
	 *
	 * @param selection
	 *            a boolean.
	 */
	public static void setSortNodesStrictlyAlphabetically(boolean selection) {
		getPreferenceStore().setValue(SORT_NODES_ALPHABETICALLY, selection);
	}

	/**
	 * <p>
	 * setStoreNavigatorState
	 * </p>
	 *
	 * @param selection
	 *            a boolean.
	 */
	public static boolean isStoreNavigatorState() {
		return getPreferenceStore().getBoolean(RESTORE_NAVIGATOR_STATE);

	}

	/**
	 * <p>
	 * setStoreNavigatorState
	 * </p>
	 *
	 * @param selection
	 *            a boolean.
	 */
	public static void setStoreNavigatorState(boolean selection) {
		getPreferenceStore().setValue(RESTORE_NAVIGATOR_STATE, selection);

	}

    /**
     * @return
     */
    public static boolean isShowUpWidgetIsDisposedMessages() {
       return getPreferenceStore().getBoolean(IS_SHOW_UP_WIDGET_IS_DISPOSED);
    }
    public static void setShowUpWidgetIsDisposedMessages(boolean selection) {
        getPreferenceStore().setValue(IS_SHOW_UP_WIDGET_IS_DISPOSED, selection);
    }

    /**
     * @return
     */
    public static boolean isShowIdInVocabularyInChecklistEditor() {
       return getPreferenceStore().getBoolean(IPreferenceKeys.CHECKLIST_ID_IN_VOCABULARY);
    }
    public static void setShowIdInVocabularyInChecklistEditor(boolean selection) {
        getPreferenceStore().setValue(CHECKLIST_ID_IN_VOCABULARY, selection);
    }

    /**
     * @return
     */
    public static boolean isShowRankInChecklistEditor() {
        return getPreferenceStore().getBoolean(IPreferenceKeys.CHECKLIST_SHOW_RANK);
    }
    public static void setShowRankInChecklistEditor(boolean selection) {
        getPreferenceStore().setValue(CHECKLIST_SHOW_RANK, selection);
    }

    /**
     * @param object
     * @param b
     * @return
     */
    public static NameDetailsConfigurator setPreferredNameDetailsConfiguration( boolean local) {
        NameDetailsConfigurator config = new NameDetailsConfigurator(true);

        CdmPreference preference = null;

        if (!local) {
            preference = getPreferenceFromDB(PreferencePredicate.NameDetailsView);
        }
        if (preference == null){
            return null;
        }

        getPreferenceStore().setValue(ALLOW_OVERRIDE_NAMEDETAILS, preference.isAllowOverride());

        //the preference value is build like this:
        //<section1>:true;<section2>:false....

        String value = preference.getValue();
        String [] sections = value.split(";");
        Map<String, Boolean> sectionMap = new HashMap<String, Boolean>();
        String[] sectionValues;
        for (String sectionValue: sections){
            sectionValues = sectionValue.split(":");
            sectionMap.put(sectionValues[0], Boolean.valueOf(sectionValues[1]));
        }



        getPreferenceStore().setValue(SHOW_SIMPLE_NAME_DETAILS_TAXON,
                (sectionMap.get("taxon")));
        config.setTaxonSectionActivated(sectionMap.get("taxon"));

        getPreferenceStore().setValue(SHOW_NAME_DETAILS_SECTION_LSID,
                (sectionMap.get("lsid")));
        config.setLSIDActivated(sectionMap.get("lsid"));

        getPreferenceStore().setValue(SHOW_NAME_DETAILS_SECTION_NOMENCLATURAL_CODE,
                (sectionMap.get("nc")));
        config.setNomenclaturalCodeActived(sectionMap.get("nc"));

        getPreferenceStore().setValue(SHOW_NAME_DETAILS_SECTION_APPENDED_PHRASE,
                (sectionMap.get("ap")));
        config.setAppendedPhraseActivated(sectionMap.get("ap"));

        getPreferenceStore().setValue(SHOW_NAME_DETAILS_SECTION_RANK,
                (sectionMap.get("rank")));
        config.setRankActivated(sectionMap.get("rank"));


        getPreferenceStore().setValue(SHOW_NAME_DETAILS_SECTION_ATOMISED_EPITHETS,
                (sectionMap.get("atomisedEpithets")));
        config.setAtomisedEpithetsActivated(sectionMap.get("atomisedEpithets"));

        getPreferenceStore().setValue(SHOW_NAME_DETAILS_SECTION_AUTHORSHIP,
                (sectionMap.get("author")));
        config.setAuthorshipSectionActivated(sectionMap.get("author"));

        getPreferenceStore().setValue(SHOW_NAME_DETAILS_SECTION_NOMENCLATURAL_REFERENCE,
                (sectionMap.get("nomRef")));
        config.setNomenclaturalReferenceSectionActivated(sectionMap.get("nomRef"));

        getPreferenceStore().setValue(SHOW_NAME_DETAILS_SECTION_NOMENCLATURAL_STATUS,
                (sectionMap.get("nomStat")));
        config.setNomenclaturalStatusSectionActivated(sectionMap.get("nomStat"));


        getPreferenceStore().setValue(SHOW_NAME_DETAILS_SECTION_PROTOLOGUE,
                (sectionMap.get("protologue")));
        config.setProtologueActivated(sectionMap.get("protologue"));

        getPreferenceStore().setValue(SHOW_NAME_DETAILS_SECTION_TYPE_DESIGNATION,
                (sectionMap.get("typeDes")));
        config.setTypeDesignationSectionActivated(sectionMap.get("typeDes"));

        getPreferenceStore().setValue(SHOW_NAME_DETAILS_SECTION_NAME_RELATIONSHIP,
                (sectionMap.get("nameRelation")));
        config.setNameRelationsSectionActivated(sectionMap.get("nameRelation"));

        return config;
    }

    public static boolean getIsDeterminiationOnlyForFieldUnit(boolean local){
        CdmPreference preference = null;

        if (!local) {
            preference = getPreferenceFromDB(PreferencePredicate.DeterminationOnlyForFieldUnits);
        }
        if (preference == null){
            return getPreferenceStore().getBoolean(DETERMINATION_ONLY_FOR_FIELD_UNITS);
        }
        return Boolean.valueOf(preference.getValue());
    }

    public static boolean getIsShowCollectingAreasInGeneralSection(boolean local){
        CdmPreference preference = null;

        if (!local) {
            preference = getPreferenceFromDB(PreferencePredicate.ShowCollectingAreasInGeneralSection);
        }
        if (preference == null){
            return getPreferenceStore().getBoolean(SHOW_COLLECTING_AREAS_IN_GENERAL_SECTION);
        }
        return Boolean.valueOf(preference.getValue());
    }

    public static boolean getIsShowTaxonAssociations(boolean local){
        CdmPreference preference = null;

        if (!local) {
            preference = getPreferenceFromDB(PreferencePredicate.ShowTaxonAssociations);
        }
        if (preference == null){
            return getPreferenceStore().getBoolean(SHOW_TAXON_ASSOCIATIONS);
        }
        return Boolean.valueOf(preference.getValue());
    }

//    public static boolean getBioCaseProvider(boolean local){
//        CdmPreference preference = null;
//
//        if (!local) {
//            preference = getPreferenceFromDB(PreferencePredicate.BioCaseProvider);
//        }
//        if (preference == null){
//            return getPreferenceStore().getBoolean(SHOW_TAXON_ASSOCIATIONS);
//        }
//        return Boolean.valueOf(preference.getValue());
//    }


    /**
     * <p>
     * setAbcdConfigurator
     * </p>
     *
     * @param preferredConfiguration
     *            a {@link eu.etaxonomy.cdm.io.specimen.abcd206.in.Abcd206ImportConfigurator}
     *            object.
     */
//    public static CdmPreference setAbcdImportConfigurationPreference(
//            Abcd206ImportConfigurator preferredConfiguration, boolean local) {
//        if (local){
//            /*
//             *
//    public static final String ABCD_IMPORT_CONFIGURATOR_DO_SIBLINGS = "eu.etaxonomy.taxeditor.abcd_import_configurator.doSiblings";
//    public static final String ABCD_IMPORT_CONFIGURATOR_IGNORE_IMPORT_OF_EXISTING_SPECIMEN= "eu.etaxonomy.taxeditor.abcd_import_configurator.ignoreImportOfExistingSpecimen";
//    public static final String ABCD_IMPORT_CONFIGURATOR_REUSE_EXISTING_TAXA_WHEN_POSSIBLE= "eu.etaxonomy.taxeditor.abcd_import_configurator.reuseExistingTaxaWhenPossible";
//    public static final String ABCD_IMPORT_CONFIGURATOR_IGNORE_AUTHORSHIP= "eu.etaxonomy.taxeditor.abcd_import_configurator.ignoreAuthorship";
//    public static final String ABCD_IMPORT_CONFIGURATOR_REMOVE_COUNTRY_FROM_LOCALITY_TEXT= "eu.etaxonomy.taxeditor.abcd_import_configurator.removeCountryFromLocalityText";
//    public static final String ABCD_IMPORT_CONFIGURATOR_ADD_MEDIA_AS_MEDIASPECIMEN= "eu.etaxonomy.taxeditor.abcd_import_configurator.addMediaAsMediaSpecimen";
//    public static final String ABCD_IMPORT_CONFIGURATOR_REUSE_EXISTING_META_DATA= "eu.etaxonomy.taxeditor.abcd_import_configurator.reuseExistingMetaData";
//    public static final String ABCD_IMPORT_CONFIGURATOR_ADD_INDIVIDUALS_ASSOCIATIONS_SUCH_AS_SPECIMEN_AND_OBSERVATIONS= "eu.etaxonomy.taxeditor.abcd_import_configurator.addIndividualsAssociationsSuchAsSpecimenAndObservations";
//    public static final String ABCD_IMPORT_CONFIGURATOR_REUSE_EXISTING_DESCRIPTIVE_GROUPS= "eu.etaxonomy.taxeditor.abcd_import_configurator.reuseExistingDescriptiveGroups";
//    public static final String ABCD_IMPORT_CONFIGURATOR_ALLOW_REUSE_OTHER_CLASSIFICATIONS= "eu.etaxonomy.taxeditor.abcd_import_configurator.allowReuseOtherClassifications";
//    public static final String ABCD_IMPORT_CONFIGURATOR_DEDUPLICATE_REFERENCES= "eu.etaxonomy.taxeditor.abcd_import_configurator.deduplicateReferences";
//    public static final String ABCD_IMPORT_CONFIGURATOR_DEDUPLICATE_CLASSIFICATIONS= "eu.etaxonomy.taxeditor.abcd_import_configurator.deduplicateClassifications";
//    public static final String ABCD_IMPORT_CONFIGURATOR_MOVE_NEW_TAXA_TO_DEFAULT_CLASSIFICATION= "eu.etaxonomy.taxeditor.abcd_import_configurator.moveNewTaxaToDefaultClassification";
//    public static final String ABCD_IMPORT_CONFIGURATOR_MAP_UNIT_ID_TOCATALOG_NUMBER= "eu.etaxonomy.taxeditor.abcd_import_configurator.mapUnitIdToCatalogNumber";
//    public static final String ABCD_IMPORT_CONFIGURATOR_MAP_UNIT_ID_TO_ACCESSION_NUMBER= "eu.etaxonomy.taxeditor.abcd_import_configurator.mapUnitIdToAccessionNumber";
//    public static final String ABCD_IMPORT_CONFIGURATOR_MAP_UNIT_ID_TO_BARCODE= "eu.etaxonomy.taxeditor.abcd_import_configurator.mapUnitIdToBarcode";
//    public static final String ABCD_IMPORT_CONFIGURATOR_OVERWRITE_EXISTING_SPECIMEN= "eu.etaxonomy.taxeditor.abcd_import_configurator.overwriteExistingSpecimens";
//
//             */
//
//            getPreferenceStore().setValue(ABCD_IMPORT_CONFIGURATOR_SOURCE_URI,
//                    getPreferenceKey(preferredConfiguration.getSourceUri()));
//        }
//        else{
//            ICdmRepository controller;
//            controller = CdmStore.getCurrentApplicationConfiguration();
//            PrefKey key = CdmPreference.NewKey(PreferenceSubject.NewDatabaseInstance(), PreferencePredicate.NomenclaturalCode);
//            CdmPreference preference = null;
//            if (controller == null){
//                return null;
//            }
//            if (preferredCode == null){
//                preference = controller.getPreferenceService().find(key);
//                if (preference == null){
//                    return null;
//                } else{
//                    int index = StringUtils.lastIndexOf(preference.getValue(), ".");
//                    UUID uuid = UUID.fromString(preference.getValue().substring(index +1, preference.getValue().length()));
//                    preferredCode = NomenclaturalCode.getByUuid(uuid);
//
//                    getPreferenceStore().setValue(CDM_NOMENCLATURAL_CODE_KEY,
//                            getPreferenceKey(preferredCode));
//                    getPreferenceStore().setValue(ALLOW_OVERRIDE_NOMENCLATURAL_CODE_KEY,preference.isAllowOverride());
//                    return preference;
//                }
//            } else{
//                preference = CdmPreference.NewInstance(PreferenceSubject.NewDatabaseInstance(), PreferencePredicate.NomenclaturalCode, preferredCode.getKey());
//                controller.getPreferenceService().set(preference);
//
//            }
//        }
//        return null;
//
//
//
//    }





}
