/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.preference.matching;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.jface.preference.ComboFieldEditor;
import org.eclipse.jface.preference.FieldEditorPreferencePage;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

import eu.etaxonomy.cdm.model.common.IdentifiableEntity;
import eu.etaxonomy.cdm.strategy.match.IMatchStrategy;
import eu.etaxonomy.cdm.strategy.match.MatchException;
import eu.etaxonomy.cdm.strategy.match.MatchMode;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author n.hoffmann
 * @created Jan 22, 2010
 * @version 1.0
 */
public abstract class AbstractMatchingPreferences<T extends IdentifiableEntity> extends FieldEditorPreferencePage implements
		IWorkbenchPreferencePage {

	/**
	 * Fields that will be excluded from the display
	 */
	private static final String ExcludePattern = "serialVersionUID|logger|allFields|ajc.*|id|updated|updatedBy|created|createdBy|uuid" +
				"|parsingProblem|problemStarts|problemEnds|PROTECTED|NOT_PROTECTED|propertyChangeSupport";

	protected IMatchStrategy matchStrategy;

	protected Class<T> clazz;

	private List<MatchMode> matchModeList;

	protected Map<String, Combo> matchModeCombos = new HashMap<String, Combo>();

    @Override
    public void init(IWorkbench workbench) {
        setPreferenceStore(PreferencesUtil.getPreferenceStore());
    }

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void createFieldEditors() {
	    if(!CdmStore.isActive()) {
            MessagingUtils.noDataSourceWarningDialog(null);
        }else{
            for(String fieldName : getFieldNames()){
                String[][] comboValues = new String[getMatchModeList().size()][2];
                for (int i=0;i<getMatchModeList().size();i++) {
                    comboValues[i][0] = getMatchModeList().get(i).name();
                    comboValues[i][1] = getMatchModeList().get(i).name();
                }
                addField(new ComboFieldEditor(this.getClass().getCanonicalName()+fieldName,
                        fieldName, comboValues,
                        getFieldEditorParent()));
            }
        }
	}

	/**
	 * Transforms the MatchMode enum into a list.
	 *
	 * @return
	 */
	private List<MatchMode> getMatchModeList(){
		if(matchModeList == null){
			matchModeList = Arrays.asList(MatchMode.values());
		}
		return matchModeList;
	}

	/**
	 * Get names of all declared fields
	 *
	 * @return
	 */
	private List<String> getFieldNames(){
		List<Field> fields = new ArrayList<Field>();

		fields = getAllFields(fields, clazz);
		List<String> fieldNames = new ArrayList<String>();

		for(Field field : fields){
			String fieldName = field.getName();
			if(! fieldName.matches(ExcludePattern)){
				fieldNames.add(fieldName);
			}
		}

		return fieldNames;
	}

	/**
	 * Get all declared fields including fields of the superclasses.
	 *
	 * @param fields a {@link java.util.List} object.
	 * @param type a {@link java.lang.Class} object.
	 * @return a {@link java.util.List} object.
	 */
	public static List<Field> getAllFields(List<Field> fields, Class<?> type) {
        fields.addAll(Arrays.asList(type.getDeclaredFields()));

	    if (type.getSuperclass() != null) {
	        fields = getAllFields(fields, type.getSuperclass());
	    }

	    return fields;
	}

	/**
	 * Returns the default match strategy for the respective class
	 *
	 * @throws eu.etaxonomy.cdm.strategy.match.MatchException if any.
	 * @return a {@link eu.etaxonomy.cdm.strategy.match.IMatchStrategy} object.
	 */
	protected abstract IMatchStrategy getDefaultMatchStrategy() throws MatchException;
}
