/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.store;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.UUID;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Display;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.service.IAgentService;
import eu.etaxonomy.cdm.api.service.IGroupService;
import eu.etaxonomy.cdm.api.service.IMediaService;
import eu.etaxonomy.cdm.api.service.INameService;
import eu.etaxonomy.cdm.api.service.IOccurrenceService;
import eu.etaxonomy.cdm.api.service.IReferenceService;
import eu.etaxonomy.cdm.api.service.IService;
import eu.etaxonomy.cdm.api.service.ITaxonService;
import eu.etaxonomy.cdm.api.service.IUserService;
import eu.etaxonomy.cdm.api.service.config.IFindTaxaAndNamesConfigurator;
import eu.etaxonomy.cdm.api.service.config.IIdentifiableEntityServiceConfigurator;
import eu.etaxonomy.cdm.model.agent.AgentBase;
import eu.etaxonomy.cdm.model.agent.TeamOrPersonBase;
import eu.etaxonomy.cdm.model.common.Group;
import eu.etaxonomy.cdm.model.common.ICdmBase;
import eu.etaxonomy.cdm.model.common.IdentifiableEntity;
import eu.etaxonomy.cdm.model.common.RelationshipBase;
import eu.etaxonomy.cdm.model.common.User;
import eu.etaxonomy.cdm.model.media.Media;
import eu.etaxonomy.cdm.model.name.NameRelationship;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase;
import eu.etaxonomy.cdm.model.reference.Reference;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.cdm.persistence.dto.UuidAndTitleCache;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.preference.IPreferenceKeys;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;

/**
 * @author n.hoffmann
 * @created Dec 8, 2010
 * @version 1.0
 */
public class SearchManager {

	public static final List NO_RESULTS = Arrays.asList(new Object[]{});

	public static final String WILDCARD = "*"; //$NON-NLS-1$

	public static int NO_COUNT = -1;


	// TODO make this configurable via preferences
	private static final int MAX_RESULTS_BEFORE_WARNING = 500;

	public List<TaxonNameBase> findNames(IIdentifiableEntityServiceConfigurator configurator, ConversationHolder conversation){

		if(checkLargeResult(CdmStore.getService(INameService.class).countByTitle(configurator))){
			List<TaxonNameBase> records = CdmStore.getService(INameService.class).findByTitle(configurator).getRecords();
			addUuidSearchResults(records, configurator, INameService.class);
            return records;
		}
		return NO_RESULTS;
	}

	public List<NameRelationship> findNameRelationships(
			IIdentifiableEntityServiceConfigurator configurator) {
		if(true){
		    //if activated again remember to add uuid search results like in other searches
			return NO_RESULTS;
		}

		List<NameRelationship> relationships = new ArrayList<NameRelationship>();
		List<RelationshipBase> all = CdmStore.getService(INameService.class).getAllRelationships(0, 0);

		for (RelationshipBase relationship : all){
			if(relationship instanceof NameRelationship){
				relationships.add((NameRelationship) relationship);
			}
		}
		return relationships;
	}

	public List<UuidAndTitleCache<IdentifiableEntity>> findTaxaAndNames(IFindTaxaAndNamesConfigurator<TaxonBase> configurator){
		return CdmStore.getService(ITaxonService.class).findTaxaAndNamesForEditor(configurator);
	}

	public List<Reference> findReferences(IIdentifiableEntityServiceConfigurator configurator){
		if(checkLargeResult(CdmStore.getService(IReferenceService.class).countByTitle(configurator))){
			List<Reference> records = CdmStore.getService(IReferenceService.class).findByTitle(configurator).getRecords();
			addUuidSearchResults(records, configurator, IReferenceService.class);
            return records;
		}
		return NO_RESULTS;
	}

	public List<AgentBase> findAgents(IIdentifiableEntityServiceConfigurator configurator){

		if(checkLargeResult(CdmStore.getService(IAgentService.class).countByTitle(configurator))){
			List<AgentBase> records = CdmStore.getService(IAgentService.class).findByTitle(configurator).getRecords();
			addUuidSearchResults(records, configurator, IAgentService.class);
            return records;
		}
		return NO_RESULTS;
	}

    /**
     * Check search string if it is a {@link UUID} and, if <code>true</code>, search for the corresponding entity.
     * @param records the list to which the search results are added
     * @param configurator the configurator holding the search string
     * @param service the service to use for searching
     */
    private <T extends ICdmBase> void addUuidSearchResults(List<T> records, IIdentifiableEntityServiceConfigurator configurator, Class<? extends IService<T>> service) {
        String titleSearchString = configurator.getTitleSearchString();
		try {
		    UUID uuid = UUID.fromString(titleSearchString);
		    T foundRecord = (T) CdmStore.getService(service).load(uuid, configurator.getPropertyPaths());
		    if(foundRecord!=null){
		        records.add(foundRecord);
		    }
		} catch (IllegalArgumentException e) {
		    //search string was no UUID
		}
    }

	@SuppressWarnings("unchecked")
	public List<TeamOrPersonBase> findTeamOrPersons(IIdentifiableEntityServiceConfigurator configurator){
	    configurator.setClazz(TeamOrPersonBase.class);
	    return (List)findAgents(configurator);
	}

	/**
	 * Searches for {@link SpecimenOrObservationBase} with the parameters specified in the
	 * {@link IIdentifiableEntityServiceConfigurator}<br>
	 * <br>
	 * Note: FieldUnits are omitted by default. See {@link #findOccurrences(IIdentifiableEntityServiceConfigurator, boolean)}
	 *
	 * @param configurator the configurator to use for the search
	 * @return a list of the SpecimenOrObservationBases matching the search parameters found
	 */
	public List<SpecimenOrObservationBase> findOccurrences(IIdentifiableEntityServiceConfigurator configurator){
	    // by default we do not show field units. This may be configured via preferences
	    return findOccurrences(configurator, PreferencesUtil.getPreferenceStore().getBoolean(IPreferenceKeys.BULK_EDITOR_OCCURRENCE_SHOW_FIELD_UNITS));
	}


	/**
	 * Searches for {@link SpecimenOrObservationBase} with the parameters specified in the
	 * {@link IIdentifiableEntityServiceConfigurator}
	 *
	 * @param configurator the configurator to use for the search
	 * @return a list of the SpecimenOrObservationBases found
	 * @param showFieldUnits if <code>true</code> then also FieldUnits are searched
	 * @return
	 */
	public List<SpecimenOrObservationBase> findOccurrences(IIdentifiableEntityServiceConfigurator<SpecimenOrObservationBase> configurator, boolean showFieldUnits){
	    List<SpecimenOrObservationBase> records = new ArrayList<SpecimenOrObservationBase>();
		final List<String> BASE_OCCURRENCE_INIT_STRATEGY = Arrays.asList(new String[] {
		        "collection", //$NON-NLS-1$
		        "descriptions", //$NON-NLS-1$
		        "identifiers", //$NON-NLS-1$
		        "derivationEvents.originals", //$NON-NLS-1$
		        "derivedFrom.originals", //$NON-NLS-1$
		        "gatheringEvent.country.representations", //$NON-NLS-1$
		        "gatheringEvent.collector", //$NON-NLS-1$
		        "gatheringEvent.locality", //$NON-NLS-1$
		        "descriptions.descriptionElements", //$NON-NLS-1$
		        "kindOfUnit", //$NON-NLS-1$
		        "amplificationResults", //$NON-NLS-1$
		        "sequences.singleReadAlignments", //$NON-NLS-1$
		        "mediaSpecimen" //$NON-NLS-1$
		});

		List<String> occurrencePropertyPaths = new ArrayList<String>();
		occurrencePropertyPaths.addAll(BASE_OCCURRENCE_INIT_STRATEGY);
		for(String propertyPath:BASE_OCCURRENCE_INIT_STRATEGY) {
		    occurrencePropertyPaths.add("derivationEvents.derivatives." + propertyPath); //$NON-NLS-1$
		}
		configurator.setPropertyPaths(occurrencePropertyPaths);

		if(checkLargeResult(CdmStore.getService(IOccurrenceService.class).countOccurrences(configurator))){
			records = CdmStore.getService(IOccurrenceService.class).findByTitle(configurator).getRecords();
		}
		addUuidSearchResults(records, configurator, IOccurrenceService.class);
		return records;
	}

	public List<User> findUsers(IIdentifiableEntityServiceConfigurator configurator){
		String userNameSearchString = sqlizeTitleSearchString(configurator);
		// TODO why are users not identifiable entities?
		List<User> records = CdmStore.getService(IUserService.class).listByUsername(userNameSearchString, null, null, null, null, null, null);
		addUuidSearchResults(records, configurator, IUserService.class);
        return records;
	}


	public List<Group> findGroups(IIdentifiableEntityServiceConfigurator configurator){
		String groupNameSearchString = sqlizeTitleSearchString(configurator);
		// TODO why are groups not identifiable entities?
		List<Group> records = CdmStore.getService(IGroupService.class).listByName(groupNameSearchString, null, null, null, null, null, null);
		addUuidSearchResults(records, configurator, IGroupService.class);
        return records;
	}


	private boolean checkLargeResult(int count) {
	    return checkLargeResult(count, MAX_RESULTS_BEFORE_WARNING);
	}

	private boolean checkLargeResult(int count, int maxBeforWarning) {
		if(count > maxBeforWarning){
			return MessageDialog.openConfirm(Display.getDefault().getActiveShell(), Messages.SearchManager_LARGE_RESULT_EXPECTED,
					String.format(Messages.SearchManager_LONG_SEARCH_WARNING, count));
		}else{
			return true;
		}
	}

	private String sqlizeTitleSearchString(IIdentifiableEntityServiceConfigurator configurator){
		return configurator.getTitleSearchString().replace(WILDCARD, "%"); //$NON-NLS-1$
	}

	public List findTaxa(IIdentifiableEntityServiceConfigurator configurator) {
		if(checkLargeResult(CdmStore.getService(ITaxonService.class).countByTitle(configurator))){
			List<TaxonBase> records = CdmStore.getService(ITaxonService.class).findByTitle(configurator).getRecords();
			addUuidSearchResults(records, configurator, ITaxonService.class);
            return records;
		}
		return NO_RESULTS;
	}

	public List findMedia(IIdentifiableEntityServiceConfigurator configurator) {
        if(checkLargeResult(CdmStore.getService(IMediaService.class).countByTitle(configurator))){
            List<Media> records = CdmStore.getService(IMediaService.class).findByTitle(configurator).getRecords();
            addUuidSearchResults(records, configurator, IMediaService.class);
            return records;
        }
        return NO_RESULTS;
    }


}
