/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.ui.combo;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map.Entry;

import org.eclipse.swt.events.SelectionEvent;

import eu.etaxonomy.cdm.model.common.IEnumTerm;
import eu.etaxonomy.cdm.model.common.OriginalSourceType;
import eu.etaxonomy.cdm.model.name.NomenclaturalCode;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationType;
import eu.etaxonomy.cdm.model.reference.ReferenceType;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.CdmPropertyChangeEvent;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;

/**
 * @author n.hoffmann
 * @created Mar 16, 2010
 * @version 1.0
 */
public class EnumComboElement<T extends IEnumTerm<T>> extends
		AbstractComboElement<T>  {

	//TODO RL
	public void setVisible(boolean b){
		combo.setVisible(b);
	}


	private static CdmEnumDataHolder[] cdmEnumDataHolders = {
		new CdmEnumDataHolder<NomenclaturalCode>(){

			@Override
			public String getName() {
				return "Nomenclatural Code";
			}

			@Override
			public Collection<NomenclaturalCode> getElements() {
				return  Arrays.asList(NomenclaturalCode.values());
			}

			@Override
			public Class<NomenclaturalCode> getClazz() {
				return NomenclaturalCode.class;
			}

		},
		new CdmEnumDataHolder<ReferenceType>(){

			@Override
			public String getName() {
				return "Reference Type";
			}

			@Override
			public Collection<ReferenceType> getElements() {
				return Arrays.asList(ReferenceType.values());
			}

			@Override
			public Class<ReferenceType> getClazz() {
				return ReferenceType.class;
			}

		},
		new CdmEnumDataHolder<SpecimenOrObservationType>(){

			@Override
			public String getName() {
				return "Specimen / Observation Type";
			}

			@Override
			public Collection<SpecimenOrObservationType> getElements() {
				return Arrays.asList(SpecimenOrObservationType.values());
			}

			@Override
			public Class<SpecimenOrObservationType> getClazz() {
				return SpecimenOrObservationType.class;
			}

		},
		new CdmEnumDataHolder<OriginalSourceType>(){

			@Override
			public String getName() {
				return "Original Source Type";
			}

			@Override
			public Collection<OriginalSourceType> getElements() {
				return Arrays.asList(OriginalSourceType.values());
			}

			@Override
			public Class<OriginalSourceType> getClazz() {
				return OriginalSourceType.class;
			}

		}

	};

	protected List<T> elementTypeList = new ArrayList<T>();

	private final Class<T> enumType;

	public EnumComboElement(CdmFormFactory formFactory,
			ICdmFormElement parentElement, Class<T> enumType, Comparator<T> comparator, int style) {
		super(formFactory, parentElement);
		this.enumType = enumType;
		label.setText(getDefaultLabelString());

		populateTypes(comparator);

		combo.addSelectionListener(this);
	}



	private String getDefaultLabelString() {
		if(enumType.equals(ReferenceType.class)){
			return "Reference Type";
		} else if (enumType.equals(NomenclaturalCode.class)){
			return "Nomenclatural Code";
		} else if(enumType.equals(OriginalSourceType.class)) {
			return "Orig. Source Type";
		} else if(enumType.equals(SpecimenOrObservationType.class)) {
			return "Spec./Obs. Type";
		}
		return "No Label";
	}

    private void populateTypes(Comparator<T> comparator){
        List<T> elementsForClass = new ArrayList<T>(getElementsForClass(enumType));
        if(comparator!=null){
            Collections.sort(elementsForClass, comparator);
        }
        // special hierarchical order for specimens
        if(enumType.equals(SpecimenOrObservationType.class)){
            LinkedHashMap<T, String> orderedTerms = AbstractUtility.orderTerms(elementsForClass);
            for(Entry<T, String> keyLabelPair : orderedTerms.entrySet()){
                elementTypeList.add(keyLabelPair.getKey());
                combo.add(keyLabelPair.getValue());
            }
        }
        // default order
        else{
            for(T enumElement : elementsForClass){
                elementTypeList.add(enumElement);
                combo.add(((Enum) enumElement).name());
            }
        }
	}

	@Override
    public void setSelection(T selection) {
		this.selection = selection;
		combo.select(elementTypeList.indexOf(selection));
	}
	/** {@inheritDoc} */
	@Override
    public void widgetSelected(SelectionEvent e) {
		selection = elementTypeList.get(combo.getSelectionIndex());
		firePropertyChangeEvent(new CdmPropertyChangeEvent(this, e));
	}

	private Collection<T> getElementsForClass(Class<T> clazz){
		CdmEnumDataHolder<T> dataHolder = getCdmEnumDataHolderForClass(clazz);
		if (dataHolder != null) {
			return dataHolder.getElements();
		}
		return null;
	}

	private CdmEnumDataHolder<T> getCdmEnumDataHolderForClass(Class<T> clazz){
		for (CdmEnumDataHolder dataHolder : cdmEnumDataHolders) {
			if (dataHolder.getClazz().equals(clazz)){
				return dataHolder;
			}
		}
		return null;
	}

	private interface CdmEnumDataHolder<T> {
		Class<T> getClazz();
		String getName();
		Collection<T> getElements();
	}
}
