/**
 *
 */
package eu.etaxonomy.taxeditor.ui.combo;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Listener;

import eu.etaxonomy.cdm.model.common.DefinedTermBase;
import eu.etaxonomy.cdm.model.common.TermType;
import eu.etaxonomy.cdm.model.common.TermVocabulary;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.preference.IPreferenceKeys;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.TermManager;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.CdmPropertyChangeEvent;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;

/**
 * @author n.hoffmann
 * @created Nov 5, 2009
 * @version 1.0
 * @param <T>
 */
public class TermComboElement<T extends DefinedTermBase>
		extends AbstractComboElement<T> {

	private T emptyElement;
	private static String EMPTY_ELEMENT_LABEL = "";

	private ArrayList<T> terms;

	private Comparator<T> termComparator;

	private final TermType termType;
	private final TermVocabulary termVocabulary;
	private final Class<T> termClass;

	private List<T> customPreferredTerms;

	private boolean useAbbrevLabel = false;
	private boolean addEmptyElement;

	public TermComboElement(CdmFormFactory formFactory,
			ICdmFormElement parentElement, TermType termType, String labelString, T selection, boolean addEmptyElement,
			int style, boolean useAbbrevLabel) {
		this(formFactory, parentElement, null, termType, null, labelString, selection, addEmptyElement, style, useAbbrevLabel);
	}

	public TermComboElement(CdmFormFactory formFactory,
	        ICdmFormElement parentElement, TermVocabulary<?> termVocabulary, String labelString, T selection, boolean addEmptyElement,
	        int style, boolean useAbbrevLabel) {
	    this(formFactory, parentElement, null, null, termVocabulary, labelString, selection, addEmptyElement, style, useAbbrevLabel);
	}

    public TermComboElement(CdmFormFactory formFactory,
            ICdmFormElement parentElement, Class<T> termClass, String labelString, T selection, boolean addEmptyElement,
            int style) {
        this(formFactory, parentElement, termClass, null, null, labelString, selection, addEmptyElement, style, false);
    }
    public TermComboElement(CdmFormFactory formFactory,
            ICdmFormElement parentElement, Class<T> termClass, String labelString, T selection, boolean addEmptyElement,
            int style, boolean useAbbrevLabel) {
        this(formFactory, parentElement, termClass, null, null, labelString, selection, addEmptyElement, style, useAbbrevLabel);

    }

	private TermComboElement(CdmFormFactory formFactory,
	        ICdmFormElement parentElement, Class<T> termClass, TermType termType, TermVocabulary<?> termVocabulary, String labelString, T selection, boolean addEmptyElement,
	        int style, boolean useAbbrevLabel) {
        super(formFactory, parentElement);

        this.termType = termType;
        this.termVocabulary = termVocabulary;
        this.termClass = termClass;
        this.addEmptyElement = addEmptyElement;
        this.useAbbrevLabel = useAbbrevLabel;
        if (labelString != null) {
            label.setText(labelString);
        }

        if(termType!=null){
            //TODO try to remove generic T and avoid classes to be used
            populateTerms((List<T>) getTermManager().getPreferredTerms(termType));
        }
        else if(termVocabulary!=null){
            populateTerms((List<T>) getTermManager().getPreferredTerms(termVocabulary));
        }
        else if(this.termClass!=null){
            populateTerms(getPreferredTerms());
        }

        combo.addSelectionListener(this);
        combo.addDisposeListener(this);
        PreferencesUtil.getPreferenceStore().addPropertyChangeListener(this);

        if (selection != null) {
            setSelection(selection);
        }
	}

	/**
	 * <p>Sets the selection of the combo to the given T object.</p>
	 * <p>Passing <code>null</code> to this method will set the selection to
	 * the empty element and effectively clear the selection</p>
	 *
	 * @param selection
	 *            a T object or <code>null</code> to clear the selection
	 */
	@Override
    public void setSelection(T selection) {
		this.selection = selection;

		Listener[] listeners = combo.getListeners(SWT.Selection);

		for (Listener listener : listeners) {
			combo.removeListener(SWT.Selection, listener);
		}
		int selectedIndex;
		if(selection == null){
			// set selection to the emptyElement
			selectedIndex = 0;
		}else{
			selectedIndex = terms.indexOf(selection);
			if (selectedIndex == -1) {
				createTermNotInPreferredTerms(selection);
				selectedIndex = terms.indexOf(selection);
			}
		}
		combo.select(selectedIndex);

		for (Listener listener : listeners) {
			combo.addListener(SWT.Selection, listener);
		}
	}

	/**
	 * Fills the combo with elements and sets up the convenience functions
	 * for selection index
	 *
	 * @param preferredTerms
	 */
	private void populateTerms(List<T> preferredTerms) {

		combo.removeAll();

		terms = new ArrayList<T>();

		int i = 1;
		int index = 0;

		if(addEmptyElement){
		    // Add an empty element for when nothing was selected yet
		    combo.add(EMPTY_ELEMENT_LABEL);
		    terms.add(emptyElement);
		}

		if (termComparator != null) {
			Collections.sort(preferredTerms, termComparator);
		}
		for (T term : preferredTerms) {
			String label = getLabel(term);
			if (label == null) {
				if (term.getTitleCache() != null) {
					label = term.getTitleCache();
					MessagingUtils.warn(getClass(),
							"Term does not have a default language representation: " + label
									+ ", " + term.getUuid());
				} else {
					label = "Unknown";
					MessagingUtils.warn(getClass(),
							"Representation Label and TitleCache empty for term: "
									+ term + ", " + term.getUuid());
				}

			}

			combo.add(label);
			terms.add(term);

			i++;
			if (selection != null) {
				if (selection.equals(term)) {
					index = i;
				}
			}
		}

		if (selection != null && index == 0) {
			createTermNotInPreferredTerms(selection);
		}

		combo.select(index);
	}

	protected List<T> getPreferredTerms(){
	    List<T> preferredTerms = new ArrayList<T>();
		if (customPreferredTerms != null){
			return customPreferredTerms;
		}
		else if(termType!=null){
		    preferredTerms = getTermManager().getPreferredTerms(termType);
		}
		else if(termVocabulary!=null){
		    preferredTerms = getTermManager().getPreferredTerms(termVocabulary);
		}
		if(termClass!=null){
		    preferredTerms = getTermManager().getPreferredTerms(termClass);
		}
		return preferredTerms;
	}

	/**
	 * May be overridden by derived classes if the desired label string does not
	 * reside in term.getLabel();
	 *
	 * @param term
	 *            a T object.
	 * @return a {@link java.lang.String} object.
	 */
	protected String getLabel(T term) {
		if (term == null){
			return "";
		}else{
			String termLabel = null;
			if (useAbbrevLabel){
				termLabel = term.getIdInVocabulary();
			}else{
				termLabel = term.getLabel(CdmStore.getDefaultLanguage());
			}
			if (termLabel == null){
			    termLabel = term.getLabel();
			}
			if(PreferencesUtil.getPreferenceStore().getBoolean(IPreferenceKeys.SHOW_VOCABULARY_ID_FOR_TERM_LABELS)
			    && term.getVocabulary()!=null){
			    String vocLabel = term.getVocabulary().getLabel(CdmStore.getDefaultLanguage());
			    if (vocLabel == null){
			        vocLabel = term.getVocabulary().getLabel();
			    }
			    termLabel += " ["+vocLabel+"]";
			}
            return termLabel;
		}
	}

	/**
	 *
	 *
	 * @param term
	 */
	private void createTermNotInPreferredTerms(T term) {
		List<T> preferredTerms = getPreferredTerms();

		preferredTerms.add(term);

		populateTerms(preferredTerms);
	}

	/** {@inheritDoc} */
	@Override
    public void widgetSelected(SelectionEvent e) {
		selection = terms.get(combo.getSelectionIndex());
		firePropertyChangeEvent(new CdmPropertyChangeEvent(this, e));
	}

	/** {@inheritDoc} */
	@Override
    public void propertyChange(PropertyChangeEvent event) {
		super.propertyChange(event);
		if (event != null
				&& PreferencesUtil.PREFERRED_TERMS_CHANGE.equals(event
						.getProperty())) {
			populateTerms(getPreferredTerms());
		}
	}

	protected TermManager getTermManager() {
		return CdmStore.getTermManager();
	}

	public int getVisibleItemCount(){
		return combo.getVisibleItemCount();
	}

	/**
	 * <p>A {@link List} of term objects may be passed to this combo box. In this case, the default behaviour
	 * of displaying the preferred terms for the T type will be overridden and the combo will only display the
	 * given terms. Also, any previous selection will be reseted.</p>
	 *
	 * <p>To return to the default of displaying the preferred terms, simply pass <code>null</code>.</p>
	 *
	 * @param terms a {@link List} of T objects or <code>null</code> for default preferred terms
	 */
	public void setTerms(List<T> terms) {
		setSelection(null);
		customPreferredTerms = terms;
		populateTerms(customPreferredTerms);
	}

	public void removeEmptyElement(){
	    if(addEmptyElement){
	        if(terms.contains(emptyElement)){
	            terms.remove(emptyElement);
	        }
	        if(Arrays.asList(combo.getItems()).contains(EMPTY_ELEMENT_LABEL)){
	            combo.remove(EMPTY_ELEMENT_LABEL);
	        }
	    }
	}
}
