/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.dialog.selection;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.UUID;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.service.IVocabularyService;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.common.TermType;
import eu.etaxonomy.cdm.model.common.TermVocabulary;
import eu.etaxonomy.cdm.model.location.NamedArea;
import eu.etaxonomy.cdm.persistence.dto.UuidAndTitleCache;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.newWizard.AbstractNewEntityWizard;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author n.hoffmann
 * @created May 11, 2010
 * @version 1.0
 */
public class NamedAreaSelectionDialog extends
		AbstractFilteredCdmResourceSelectionDialog<NamedArea> {

	private class IncludeNamedAreaVocabulary extends Action {
		private final TermVocabulary<NamedArea> vocabulary;

		/**
		 * Creates a new instance of the class.
		 */
		public IncludeNamedAreaVocabulary(TermVocabulary<NamedArea> vocabulary) {
			super(vocabulary.getTitleCache(), IAction.AS_CHECK_BOX);
			this.vocabulary = vocabulary;
		}

		@Override
        public void run(){
			if(isChecked()){
				selectedVocabularies.add(vocabulary);
			}else{
				selectedVocabularies.remove(vocabulary);
			}
			PreferencesUtil.getPreferenceStore().setValue(getPrefKey(vocabulary), !isChecked());
			initModel();
		}
	}

	protected Collection<TermVocabulary> selectedVocabularies;
    protected ArrayList<TermVocabulary> preselectedVocabularies;
    private Object preferenceID;


	/**
	 * Creates a filtered selection dialog to select a named area.
	 *
	 * @param shell
	 * 				The shell for displaying this widget
	 * @param namedArea
	 * 				A namedArea that should be selected when the dialog opens
	 * @param conversation a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
	 * @param preferenceId a class which is used for generating the preference key so that every 
	 * dialogs can be grouped to have their own preferences depending on this id
	 * @param preselectedVocabularyUuids the {@link UUID}s of the pre-selected vocabularies
	 * @return a {@link eu.etaxonomy.cdm.model.location.NamedArea} object.
	 */
	public static NamedArea select(Shell shell, ConversationHolder conversation, NamedArea namedArea, String preferenceId, UUID... preselectedVocabularyUuids) {
		NamedAreaSelectionDialog dialog = new NamedAreaSelectionDialog(shell, conversation,
				"Choose an area", false, namedArea, preferenceId, preselectedVocabularyUuids);
		return getSelectionFromDialog(dialog);
	}

	protected NamedAreaSelectionDialog(Shell shell, ConversationHolder conversation, String title, boolean multi, NamedArea namedArea, Object preferenceId, UUID... preselectedVocabularyUuids) {
		super(shell, conversation, title, multi, NamedAreaSelectionDialog.class.getCanonicalName(), namedArea);
		this.preferenceID = preferenceId;
		preselectedVocabularies = new ArrayList<TermVocabulary>();
		for(int i=0;i<preselectedVocabularyUuids.length;i++){
			TermVocabulary preselectedVocabulary = CdmStore.getService(IVocabularyService.class).find(preselectedVocabularyUuids[i]);
			preselectedVocabularies.add(preselectedVocabulary);
		}
		Collection<TermVocabulary> tempSelectedVocabularies = new HashSet<>();
		for(TermVocabulary vocabulary:selectedVocabularies){
			if(preselectedVocabularies.contains(vocabulary)
					|| !PreferencesUtil.getPreferenceStore().getBoolean(getPrefKey(vocabulary))){
				tempSelectedVocabularies.add(vocabulary);
			}
		}
		selectedVocabularies = tempSelectedVocabularies;
		initModel();//re-init to consider pre-selected vocabularies
	}

	/** {@inheritDoc} */
	@Override
	protected void fillViewMenu(IMenuManager menuManager) {

		super.fillViewMenu(menuManager);

		for(TermVocabulary<NamedArea> vocabulary : getAvailableVocabularies()){
			IncludeNamedAreaVocabulary action = new IncludeNamedAreaVocabulary(vocabulary);
			menuManager.add(action);
			if(preselectedVocabularies.contains(vocabulary)) {
			    action.setChecked(true);
			}
			else{
				action.setChecked(!PreferencesUtil.getPreferenceStore().getBoolean(getPrefKey(vocabulary)));
			}
		}
	}
	
	private String getPrefKey(TermVocabulary vocabulary){
		return "hide_"+NamedAreaSelectionDialog.class.getCanonicalName()+vocabulary.getUuid()+preferenceID;
	}

	/** {@inheritDoc} */
	@Override
	protected NamedArea getPersistentObject(UUID uuid) {
		for(TermVocabulary<NamedArea> vocabulary : selectedVocabularies){
			for(Object object : vocabulary.getTerms()){
				CdmBase cdmBaseObject = (CdmBase) object;
				if(uuid.equals(cdmBaseObject.getUuid())){
					return CdmBase.deproxy(cdmBaseObject, NamedArea.class);
				}
			}
		}
		return null;
	}

	/** {@inheritDoc} */
	@Override
	protected void init() {
		selectedVocabularies = getAvailableVocabularies();
	}

	private List<TermVocabulary> getAvailableVocabularies(){
		List<TermVocabulary> vocabularies = CdmStore.getService(IVocabularyService.class).listByTermType(TermType.NamedArea, true, null, null, null, null);
		return vocabularies;
	}

	/** {@inheritDoc} */
	@Override
	protected void initModel() {

		Set<NamedArea> terms = new HashSet<NamedArea>();
		for(TermVocabulary<NamedArea> vocabulary : selectedVocabularies){
			terms.addAll(vocabulary.getTermsOrderedByLabels(CdmStore.getDefaultLanguage()));
		}

		if(model == null){
			model = new ArrayList<UuidAndTitleCache<NamedArea>>();
		}
		model.clear();
		for(Object areaObject : terms){
			NamedArea area = (NamedArea) HibernateProxyHelper.deproxy(areaObject);
			UuidAndTitleCache<NamedArea> element = new UuidAndTitleCache<NamedArea>(NamedArea.class, area.getUuid(), area.getId(), getTitle(area));
			model.add(element);
		}
	}

	/** {@inheritDoc} */
	@Override
	protected Control createExtendedContentArea(Composite parent) {
		return null;
	}

	/** {@inheritDoc} */
	@Override
	protected String getTitle(NamedArea namedArea) {
		try {
			String result = NamedArea.labelWithLevel(namedArea, CdmStore.getDefaultLanguage());
			return result;
		} catch (Exception e) {
			MessagingUtils.error(NamedAreaSelectionDialog.class, "Error occurred when trying retrieve title for Named Area: " + namedArea.getUuid(), e);
			return namedArea.getTitleCache();
		}
	}

	/** {@inheritDoc} */
	@Override
	protected AbstractNewEntityWizard getNewEntityWizard(String parameter) {
		return null;
	}

	/** {@inheritDoc} */
	@Override
	protected String getNewWizardLinkText() {
		return null;
	}
}
