/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.element;

import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.forms.widgets.TableWrapLayout;
import org.joda.time.IllegalFieldValueException;
import org.joda.time.Partial;

import eu.etaxonomy.cdm.model.common.TimePeriod;

/**
 * <p>PartialElement class.</p>
 *
 * @author n.hoffmann
 * @created Mar 31, 2010
 */
public class PartialElement extends AbstractCdmFormElement implements ISelectable {

	private Label label;
	private NumberWithLabelElement number_day;
	private NumberWithLabelElement number_month;
	private NumberWithLabelElement number_year;
	private Partial partial;

	/**
	 * <p>Constructor for PartialElement.</p>
	 *
	 * @param formFactory a {@link eu.eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 * @param formElement a {@link eu.etaxonomy.taxeditor.ui.element.forms.ICdmFormElement} object.
	 * @param labelString a {@link java.lang.String} object.
	 * @param style a int.
	 */
	public PartialElement(CdmFormFactory formFactory,
			ICdmFormElement formElement, String labelString, int style) {
		super(formFactory, formElement);

		TableWrapLayout layout = LayoutConstants.LAYOUT(7, false);
		layout.horizontalSpacing = 5;
        formElement.getLayoutComposite().setLayout(layout);

		label = formFactory.createLabel(getLayoutComposite(), labelString);
		addControl(label);

		number_year = formFactory.createNumberTextWithLabelElement(this, "Year", null, style);
		number_year.setLimits(4, 0, 9999);

		number_month = formFactory.createNumberTextWithLabelElement(this, "Month", null, style);
		number_month.setLimits(2, 1, 12);

		number_day = formFactory.createNumberTextWithLabelElement(this, "Day", null, style);
		number_day.setLimits(2, 1, 31);


		formFactory.addPropertyChangeListener(this);
	}

	/**
	 * <p>Setter for the field <code>partial</code>.</p>
	 *
	 * @param partial a {@link org.joda.time.Partial} object.
	 */
	public void setPartial(Partial partial){
		this.partial = partial;

		if(partial != null){
			number_day.setNumber(TimePeriod.getPartialValue(partial, TimePeriod.DAY_TYPE));
			number_month.setNumber(TimePeriod.getPartialValue(partial, TimePeriod.MONTH_TYPE));
			number_year.setNumber(TimePeriod.getPartialValue(partial, TimePeriod.YEAR_TYPE));
		}
	}

	/**
	 * <p>Getter for the field <code>partial</code>.</p>
	 *
	 * @return a {@link org.joda.time.Partial} object.
	 */
	public Partial getPartial(){
		return partial;
	}

	/** {@inheritDoc} */
	@Override
	public void propertyChange(PropertyChangeEvent event) {
		if(event == null){
			return;
		}

		Object eventSource = event.getSource();

		if(getElements().contains(eventSource)){
			if(event instanceof CdmPropertyChangeEvent){
				if(((CdmPropertyChangeEvent) event).hasException()){
					handleException((CdmPropertyChangeEvent) event);
					return;
				}
			}
			handleEvent(eventSource);
		}
	}

	/**
	 * @param event
	 */
	private void handleException(CdmPropertyChangeEvent event) {
		firePropertyChangeEvent(new CdmPropertyChangeEvent(this, event.getException()));
	}

	private void handleEvent(Object eventSource){
	    if (eventSource instanceof NumberWithLabelElement){
			try{
		    	if(eventSource == number_month && number_month.getException()==null){
			        partial = TimePeriod.setPartialField(partial, number_month.getInteger(), TimePeriod.MONTH_TYPE);
			    }else if(eventSource == number_day && number_day.getException()==null){
	                partial = TimePeriod.setPartialField(partial, number_day.getInteger(), TimePeriod.DAY_TYPE);
			    }else if(eventSource == number_year && number_year.getException()==null){
			        partial = TimePeriod.setPartialField(partial, number_year.getInteger(), TimePeriod.YEAR_TYPE);
			    }
			} catch (IndexOutOfBoundsException | IllegalFieldValueException e) {
				((NumberWithLabelElement)eventSource).text.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_RED));
			}
	    }
	    firePropertyChangeEvent(new CdmPropertyChangeEvent(this, null));
	}

	/** {@inheritDoc} */
	@Override
	public void setBackground(Color color) {
		label.setBackground(color);
	}

	@Override
	public void setSelected(boolean selected) {
		setBackground(selected ? SELECTED : getPersistentBackground());
	}
}
