/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.ui.section;

import java.util.ArrayList;
import java.util.Collection;
import java.util.EnumSet;
import java.util.Observable;
import java.util.Observer;

import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.widgets.Composite;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.cdm.persistence.hibernate.permission.CRUD;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.LoginManager;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.ui.element.AbstractCdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.CdmPropertyChangeEvent;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.IEnableableFormElement;

/**
 *
 * @author n.hoffmann
 * @created Feb 26, 2010
 * @version 1.0
 */
public abstract class AbstractCdmDetailElement<T> extends AbstractCdmFormElement implements ICdmDetailElement<T>, Observer {

	private T entity;

	private boolean irrelevant;

	private boolean enabled;

	private EnumSet<CRUD> requiredCrud = null;

	public AbstractCdmDetailElement(CdmFormFactory formFactory,
			ICdmFormElement formElement) {
		super(formFactory, formElement);
		// register as property change listener
		formFactory.addPropertyChangeListener(this);
		CdmStore.getLoginManager().addObserver(this);

		getLayoutComposite().getParent().addDisposeListener(new DisposeListener() {

            @Override
            public void widgetDisposed(DisposeEvent e) {
                CdmStore.getLoginManager().deleteObserver(AbstractCdmDetailElement.this);
            }
        });
	}

	/**
	 * Sets the entity and updates controls
	 *
	 * @param entity
	 *            a T object.
	 */
	@Override
    public void setEntity(T entity) {
        this.entity = entity;
        // the id is always 0 if the entity was not yet saved, so it is new in this case
        if(getEntity() == null || StoreUtil.getCdmEntity(getEntity()).getId() == 0) {
            // new entity, not yet saved
            requiredCrud = EnumSet.of(CRUD.CREATE);
        } else {
            requiredCrud = EnumSet.of(CRUD.UPDATE);
        }
        updateContent();
	}

	@Override
    public T getEntity() {
		return entity;
	}

	/**
	 * Updates all widgets to display the latest data
	 */
	protected void updateContent() {
		removeElements();
		createControls(this, entity, SWT.WRAP);
		reflowParentScrolledForm(true);

		updateControlStates();
	}

    @Override
    public void setSelected(boolean selected) {
		Composite section = getLayoutComposite().getParent();
		section.setBackground(selected ? SELECTED : getPersistentBackground());
	}

	/**
	 * Sets all field to the given enablement state except the objects in
	 * collection except
	 *
	 * @see eu.etaxonomy.taxeditor.ui.element.IEnableableFormElement#setEnabled(boolean)
	 * @param enabled
	 *            a boolean.
	 * @param except
	 *            a {@link java.util.Collection} object.
	 */
	public void setEnabled(boolean enabled, Collection<Object> except) {
	    this.enabled = enabled;
		for (ICdmFormElement element : getElements()) {
			if (element instanceof IEnableableFormElement) {
				IEnableableFormElement enableableElement = (IEnableableFormElement) element;
				if (except != null && except.contains(enableableElement)) {
					// enableableElement.setEnabled(enabled);
				} else {
					enableableElement.setEnabled(enabled);
				}
			}
		}
	}

	@Override
	public boolean isEnabled() {
	    return enabled;
	}

	/**
	 * <p>
	 * Setter for the field <code>irrelevant</code>.
	 * </p>
	 *
	 * @param irrelevant
	 *            a boolean.
	 * @param except
	 *            a {@link java.util.Collection} object.
	 */
	public void setIrrelevant(boolean irrelevant, Collection<Object> except) {
		// logger.warn("Setting " + this.getClass().getSimpleName() +
		// " to irrelevant state: " + irrelevant);
		for (ICdmFormElement element : getElements()) {
			if (element instanceof IEnableableFormElement) {
				IEnableableFormElement relevantElement = (IEnableableFormElement) element;
				if (except != null && except.contains(relevantElement)) {
					// do nothing
					// logger.warn("not setting elements irrelevance: " +
					// relevantElement.getClass().getSimpleName() + " in :" +
					// this.getClass().getSimpleName());
				} else {
					relevantElement.setIrrelevant(irrelevant);
				}
			}
		}
	}

	/** {@inheritDoc} */
	@Override
    public void setEnabled(boolean enabled) {
		setEnabled(enabled, null);
	}

	/** {@inheritDoc} */
	@Override
    public void setIrrelevant(boolean irrelevant) {
		this.irrelevant = irrelevant;
		setIrrelevant(irrelevant, null);
	}

	/**
	 * Create all content elements
	 *
	 * @param formElement
	 *            a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement}
	 *            object.
	 * @param entity
	 *            a T object.
	 * @param style
	 *            a int.
	 */
	protected abstract void createControls(ICdmFormElement formElement,
			T entity, int style);

	/** {@inheritDoc} */
	@Override
	public void propertyChange(PropertyChangeEvent event) {
		if (event == null) {
			return;
		}

		handleTitleCacheRelevantChange(event);

		Object eventSource = event.getSource();

		if (eventSource instanceof ICdmFormElement) {
			ICdmFormElement eventElement = (ICdmFormElement) eventSource;
			ICdmFormElement eventElementContainer = eventElement
					.getParentElement();

			if (eventElementContainer == this) {
				if (event instanceof CdmPropertyChangeEvent) {
					if (((CdmPropertyChangeEvent) event).hasException()) {
						handleException((CdmPropertyChangeEvent) event);
						return;
					}
				}
				// call specific handler implementation
				handleEvent(eventSource);
				// refire the event
				firePropertyChangeEvent(this, event);
				// update possible parents
				updateParentSection();
			}
		}
	}

	protected void handleTitleCacheRelevantChange(PropertyChangeEvent event) {
		boolean isRelevant = getEntity() instanceof TaxonNameBase
				|| getEntity() instanceof TaxonBase;
		isRelevant &= this instanceof AbstractIdentifiableEntityDetailElement;

		if (isRelevant) {
			((AbstractIdentifiableEntityDetailElement) this)
					.updateToggleableCacheField();
			updateParentSection();
		}
	}

	private void updateParentSection() {
		if (getParentElement() instanceof AbstractCdmDetailSection) {
			((AbstractCdmDetailSection) getParentElement()).updateTitle();
		}
		reflowParentScrolledForm(true);
	}

	/**
	 * Gets called when the source of the event is a child widget of this
	 * widget.
	 *
	 * @param eventSource
	 *            a {@link java.lang.Object} object.
	 */
	public abstract void handleEvent(Object eventSource);

	public void handleException(CdmPropertyChangeEvent event) {
		// override this in subclasses if you want to deal with the error
		MessagingUtils.error(event.getSource().getClass(), event.getException());
	}

	public boolean isIrrelevant() {
		return irrelevant;
	}

	@Override
    public ConversationHolder getConversationHolder() {
		if (getParentElement() instanceof IConversationEnabled) {
			return ((IConversationEnabled) getParentElement())
					.getConversationHolder();
		}
		throw new IllegalArgumentException(
				"Parent element should be IConversationEnabled");
	}

	/** {@inheritDoc} */
	@Override
    public void update(CdmDataChangeMap changeEvents) {
	}

	@Override
    public void update(Observable o, Object arg){
        if(o instanceof LoginManager){
            updateControlStates();
        }
    }


    protected void updateControlStates(){
        Collection<Object> except = new ArrayList<Object>();
        for(ICdmFormElement formElement:getElements()){
            if(formElement instanceof IEnableableFormElement && !((IEnableableFormElement) formElement).isEnabled()){
                except.add(formElement);
            }
        }
        setEnabled(getEntity() != null && CdmStore.currentAuthentiationHasPermission(StoreUtil.getCdmEntity(getEntity()), requiredCrud), except);
    }
}
