/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.ui.section.classification;

import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.model.name.NonViralName;
import eu.etaxonomy.cdm.model.reference.Reference;
import eu.etaxonomy.cdm.model.taxon.ITaxonTreeNode;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.newWizard.IWizardPageListener;
import eu.etaxonomy.taxeditor.parser.ParseHandler;
import eu.etaxonomy.taxeditor.ui.element.AbstractCdmEntityWizardPage;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;

/**
 * <p>
 * TaxonNodeWizardPage class.
 * </p>
 *
 * @author n.hoffmann
 * @created Sep 15, 2009
 * @version 1.0
 */
public class TaxonNodeWizardPage extends AbstractCdmEntityWizardPage<ITaxonTreeNode> implements Listener{

    private final IWizardPageListener wizardPageListener;
    private final boolean isCreateNew;

    /**
	 * <p>
	 * Constructor for TaxonNodeWizardPage.
	 * </p>
	 *
	 * @param formFactory
	 *            a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory}
	 *            object.
	 * @param conversation
	 *            a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder}
	 *            object.
	 * @param entity
	 *            a {@link eu.etaxonomy.cdm.model.taxon.ITaxonTreeNode} object.
     * @param closeOnEnterListener
	 */
	public TaxonNodeWizardPage(CdmFormFactory formFactory,
			ConversationHolder conversation, ITaxonTreeNode entity, IWizardPageListener closeOnEnterListener, boolean isCreateNew) {
		super(formFactory, conversation, entity);
		this.wizardPageListener = closeOnEnterListener;
		setTitle("Edit taxon node");
		this.isCreateNew = isCreateNew;

	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * eu.etaxonomy.taxeditor.forms.AbstractCdmEntityWizardPage#createElement
	 * (eu.etaxonomy.taxeditor.forms.ICdmFormElement)
	 */
	/** {@inheritDoc} */
	@Override
	public TaxonNodeDetailElement createElement(ICdmFormElement rootElement) {
	    TaxonNodeDetailElement detailElement = formFactory.createTaxonNodeDetailElement(rootElement, this.isCreateNew);
		detailElement.setEntity(entity);
		formFactory.addPropertyChangeListener(this);
		if (isCreateNew){
		    detailElement.getMainControl().addListener(SWT.KeyDown, this);
		}
		return detailElement;
	}

	/**
	 * <p>
	 * openInEditor
	 * </p>
	 *
	 * @return a boolean.
	 */
	public boolean openInEditor() {
		return ((TaxonNodeDetailElement) getDetailElement()).isOpenInEditor();
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * org.eclipse.jface.util.IPropertyChangeListener#propertyChange(org.eclipse
	 * .jface.util.PropertyChangeEvent)
	 */
	/** {@inheritDoc} */
	@Override
	public void propertyChange(PropertyChangeEvent event) {
		checkComplete();
	}

	@Override
	protected void checkComplete() {
		TaxonNodeDetailElement detailElement = (TaxonNodeDetailElement) getDetailElement();

		if (detailElement.getClassification() == null) {
			setMessage("No classification set.", WARNING);
		}
		else if(isCreateNew ){
		    if(CdmUtils.isBlank(detailElement.getTaxonName())){
		        setMessage("No taxon name set.", ERROR);
		    } else{
		        setMessage(null);
	            setPageComplete(true);
	            return;
		    }
		}
		else if(!detailElement.isComplete()){
		    setMessage("Not all required fields are filled.", ERROR);
		}
		else {
			setMessage(null);
			setPageComplete(true);
			return;
		}

		setPageComplete(false);
	}

	/**
	 * <p>
	 * Getter for the field <code>parentTreeNode</code>.
	 * </p>
	 *
	 * @return a {@link eu.etaxonomy.cdm.model.taxon.ITaxonTreeNode} object.
	 */
	public ITaxonTreeNode getParentTreeNode() {
		return ((TaxonNodeDetailElement) getDetailElement())
				.getParentTreeNode();
	}

	public Taxon createTaxon() {
		if (((TaxonNodeDetailElement) getDetailElement()).getTaxon() == null){
			String taxonName = ((TaxonNodeDetailElement) getDetailElement()).getTaxonName();
			Reference reference = ((TaxonNodeDetailElement) getDetailElement()).getReference();
			NonViralName parseReferencedName = ParseHandler.parseReferencedName(taxonName, null);
			return Taxon.NewInstance(parseReferencedName, reference);
		} else{
			return ((TaxonNodeDetailElement) getDetailElement()).getTaxon();
		}
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see eu.etaxonomy.taxeditor.forms.AbstractCdmEntityWizardPage#dispose()
	 */
	/** {@inheritDoc} */
	@Override
	public void dispose() {
		formFactory.removePropertyChangeListener(this);
		super.dispose();
	}

    @Override
    public void handleEvent(Event event) {
        TaxonNodeDetailElement detailElement = (TaxonNodeDetailElement) getDetailElement();
        if(event.widget == detailElement.getMainControl()
                && event.keyCode == SWT.CR && isPageComplete()){
            wizardPageListener.close();
        }
    }

}
