/**
* Copyright (C) 2014 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.media;

import java.net.URI;
import java.util.List;
import java.util.Set;

import org.apache.sanselan.ImageReadException;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Label;

import eu.etaxonomy.cdm.common.UriUtils;
import eu.etaxonomy.cdm.common.media.ImageInfo;
import eu.etaxonomy.cdm.model.media.ImageFile;
import eu.etaxonomy.cdm.model.media.Media;
import eu.etaxonomy.cdm.model.media.MediaRepresentation;
import eu.etaxonomy.cdm.model.media.MediaRepresentationPart;
import eu.etaxonomy.cdm.model.media.MediaUtils;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.preference.IPreferenceKeys;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.ImageElement;
import eu.etaxonomy.taxeditor.ui.element.LayoutConstants;
import eu.etaxonomy.taxeditor.ui.element.UriWithLabelElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailElement;

/**
 * @author pplitzner
 * @date 25.08.2014
 *
 */
public class MediaDetailElement extends AbstractCdmDetailElement<Media>{

    private UriWithLabelElement textUri;
    private MediaRepresentationSection section_mediaRepresentation;

    private boolean isAdvancedMediaView;
    private MediaRepresentationPart singleMediaRepresentationPart;
    private ICdmFormElement parentFormElement;
    private int style;

    /**
     * Used to store the URI even if it is invalid and thus cannot be stored in CDM
     */
    private String uriBuffer;
    private ImageElement element_image;
    private Label lblNoImage;

    public MediaDetailElement(CdmFormFactory formFactory, ICdmFormElement formElement) {
        super(formFactory, formElement);
    }

    @Override
    protected void createControls(ICdmFormElement formElement, Media entity, int style) {
        this.parentFormElement = formElement;
        this.style = style;
    }

    @Override
    public void setEntity(Media entity) {
        super.setEntity(entity);
        if(singleMediaRepresentationPart==null){
            singleMediaRepresentationPart = MediaUtils.initFirstMediaRepresentationPart(entity, true);
        }
        initIsAdvancedMediaView();
        showAdvancedView();
    }

    @Override
    public void handleEvent(Object eventSource){
        if(eventSource==textUri){
            textUri.setBackground(getPersistentBackground());
            URI uri = textUri.parseText();
            singleMediaRepresentationPart.setUri(uri);
            if(uri==null){
                uriBuffer=textUri.getText();
            }
            else{
                createImageElement(uri);
            }
        }
    }

    private void createImageElement(URI uri) {
        ImageInfo imageInfo;
        try {
            if(uri == null){
                return;
            }
            //first check if uri refers to an actual (non-image) file
            UriUtils.getInputStream(uri);// will fail with a FileNotFoundException if not
            imageInfo = ImageInfo.NewInstance(uri, 10000);//will fail when it is no image file
            singleMediaRepresentationPart.setSize((int) imageInfo.getLength());
            if(singleMediaRepresentationPart instanceof ImageFile){
                ((ImageFile) singleMediaRepresentationPart).setHeight(imageInfo.getHeight());
                ((ImageFile) singleMediaRepresentationPart).setWidth(imageInfo.getWidth());
            }
            singleMediaRepresentationPart.getMediaRepresentation().setMimeType(imageInfo.getMimeType());
            singleMediaRepresentationPart.getMediaRepresentation().setSuffix(imageInfo.getSuffix());
            element_image = formFactory.createImageElement(parentFormElement, uri, style);
            element_image.initImageUri(uri);
            element_image.loadImage();
    		disposeErrorLabel();
        } catch (Exception e){
        	if(e.getCause()!=null && e.getCause().getClass().equals(ImageReadException.class)){
        		disposeErrorLabel();
        		handleException(uri, "No preview available for this file type");
        	}
        	else{
        		disposeErrorLabel();
        		handleException(uri, "No file found");
        	}
        }
    }

	private void disposeErrorLabel() {
		if(lblNoImage!=null){
		    lblNoImage.dispose();
		}
		lblNoImage = null;
	}

    public void toggleAdvancedMediaView() {
        if (getEntity().getRepresentations() != null
                && (getEntity().getRepresentations().size() > 1 ||
                        (getEntity().getRepresentations().size() == 1
                        && getEntity().getRepresentations().iterator().next().getParts().size() > 1))) {
            MessagingUtils.informationDialog("Toggling not possible",
                    "Media has consists of multiple representations or representatio parts");
            // toggling is only possible if there are no more than one
            // MediaRepresentation resp. MediaRepresentationParts
            return;
        }
        isAdvancedMediaView = !isAdvancedMediaView;
        showAdvancedView();
        reflowParentScrolledForm(true);
    }

    private void showAdvancedView() {
        if(isAdvancedMediaView){
            if(textUri!=null){
                removeElementsAndControls(textUri);
            }
            if(element_image!=null){
                element_image.dispose();
                element_image = null;
            }
            section_mediaRepresentation = formFactory.createMediaRepresentationSection(getConversationHolder(), parentFormElement, style);
            section_mediaRepresentation.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
            section_mediaRepresentation.setEntity(getEntity());
            //set buffered uri as text if uri had parsing problems in simple view
            if(uriBuffer!=null){
                section_mediaRepresentation.getLayoutComposite().getChildren();
            }
        }
        else{
            if(section_mediaRepresentation!=null){
                removeElementsAndControls(section_mediaRepresentation);
            }
            textUri = formFactory.createUriWithLabelElement(parentFormElement, "Media URI", null, style);
            URI uri = singleMediaRepresentationPart.getUri();
            textUri.setParsedText(uri);
            //set buffered uri as text if uri had parsing problems in advanced view
            if(uri==null && uriBuffer!=null){
                textUri.setText(uriBuffer);
                textUri.parseText();
            }
            textUri.getLayoutComposite().layout();

            createImageElement(singleMediaRepresentationPart.getUri());
        }
    }

    private void initIsAdvancedMediaView(){
        if(PreferencesUtil.getPreferenceStore().getBoolean(IPreferenceKeys.SHOW_ADVANCED_MEDIA_SECTION)){
            isAdvancedMediaView = true;
            return;
        }
        Set<MediaRepresentation> representations = getEntity().getRepresentations();
        if(representations.size()>1){
            isAdvancedMediaView = true;
            return;
        }
        if(representations.size()==1){
            List<MediaRepresentationPart> parts = representations.iterator().next().getParts();
            if(parts.size()>1){
                isAdvancedMediaView =  true;
                return;
            }
        }
        isAdvancedMediaView =  false;
    }

    private void handleException(URI uri, String labelText) {
        if(element_image!=null){
            element_image.unloadImage();
            element_image.loadImage();
            element_image.dispose();
        }
        if(lblNoImage==null){
			lblNoImage = formFactory.createLabel(getLayoutComposite(), labelText);
            lblNoImage.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
            lblNoImage.setAlignment(SWT.CENTER);
        }
    }

    public boolean isAdvancedMediaView() {
        return isAdvancedMediaView;
    }

    public void setUriBuffer(String uriBuffer) {
        this.uriBuffer = uriBuffer;
    }

    public String getUriBuffer() {
        return uriBuffer;
    }

}
