/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.view;

import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.forms.IFormPart;
import org.eclipse.ui.forms.ManagedForm;
import org.eclipse.ui.forms.widgets.ScrolledForm;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.taxeditor.ui.element.AbstractFormSection;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.LayoutConstants;
import eu.etaxonomy.taxeditor.ui.element.RootElement;
import eu.etaxonomy.taxeditor.view.detail.CdmSectionPart;

/**
 * @author n.hoffmann
 * @created Feb 9, 2010
 * @version 1.0
 */
public abstract class AbstractCdmDataViewer extends Viewer implements IConversationEnabled{

	protected ManagedForm managedForm;

	protected CdmFormFactory formFactory;

	protected ScrolledForm scrolledForm;

	private final Composite body;

	protected RootElement rootElement;

	//TODO: create a super class for this?
	private Object input;

	private final AbstractCdmViewPart viewPart;


	public AbstractCdmDataViewer(Composite parent, AbstractCdmViewPart viewPart) {
		this.viewPart = viewPart;

		managedForm = new ManagedForm(parent){

			@Override
			public void dirtyStateChanged() {
				markViewPartDirty();
			}
		};

		createFormFactory();

		scrolledForm = managedForm.getForm();

		body = scrolledForm.getBody();

		body.setLayout(LayoutConstants.LAYOUT());

		rootElement = new RootElement(formFactory, body);
	}

	protected void createFormFactory() {
		if(formFactory != null){
			formFactory.dispose();
			formFactory = null;
		}
		formFactory = new CdmFormFactory(Display.getCurrent(), this);
	}

	protected void markViewPartDirty(){
		viewPart.changed(input);
	}

	protected AbstractCdmViewPart getViewPart() {
		return viewPart;
	}

	/** {@inheritDoc} */
	@Override
	public Control getControl() {
		if(body.isDisposed()){
			return null;
		}
		for(Control child : body.getChildren()){
			return child;
		}

		return body;
	}
	
	/** {@inheritDoc} */
	@Override
	public void setInput(Object input) {
		this.input = input;
		// reset selection
		setSelection(new StructuredSelection(input));
		refresh();
	}

	/** {@inheritDoc} */
	@Override
	public Object getInput() {
		return input;
	}

	/** {@inheritDoc} */
	@Override
	public void refresh() {
		showParts();

		managedForm.setInput(input);

		managedForm.refresh();

		managedForm.reflow(true);
	}

	public void layout(){
		body.layout();
	}

	protected abstract void showParts();

	protected void destroySections() {
		for (IFormPart formPart : managedForm.getParts()){
			removePart((CdmSectionPart<?>) formPart);
		}

		managedForm.setInput(null);

		formFactory.destroyElement(rootElement);

		createFormFactory();

		rootElement = new RootElement(formFactory, body);

		for(Control control : body.getChildren()){
			control.dispose();
			control = null;
		}
	}

	public void setFocus(){
		// we have to set focus to a control of this viewer
		// otherwise, after opening a dialog from the details, the focus will not be
		// given back to the details view but to the editor
		for(Control child : body.getChildren()){
			child.setFocus();
			break;
		}
	}

	public void reflow(){
		managedForm.reflow(true);
	}

	protected void removePart(CdmSectionPart<?> sectionPart){
		managedForm.removePart(sectionPart);
		formFactory.removePropertyChangeListener(sectionPart);
	}

	protected void addPart(AbstractFormSection<?> section){
		CdmSectionPart<?> sectionPart = new CdmSectionPart<>(section);
		managedForm.addPart(sectionPart);
		formFactory.addPropertyChangeListener(sectionPart);
	}

	@Override
    public ConversationHolder getConversationHolder() {
		return viewPart.getConversationHolder();
	}

}
