/**
* Copyright (C) 2013 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.view.dataimport;

import java.util.ArrayList;
import java.util.Collection;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.viewers.CheckboxTableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.part.ViewPart;
import org.eclipse.wb.swt.ResourceManager;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.ext.occurrence.OccurenceQuery;
import eu.etaxonomy.cdm.model.taxon.Classification;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.model.IContextListener;
import eu.etaxonomy.taxeditor.model.IPartContentHasDetails;
import eu.etaxonomy.taxeditor.model.IPartContentHasFactualData;
import eu.etaxonomy.taxeditor.model.IPartContentHasSupplementalData;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.ui.dialog.selection.SelectionDialogFactory;

/**
 * View which shows a list of "data" that can be imported into the CDM
 * @author pplitzner
 * @date Sep 3, 2014
 *
 * @param <T> the CDM type that will be handled by this view
 */

public abstract class DataImportView<T> extends ViewPart implements IPartContentHasFactualData,
IConversationEnabled, IPartContentHasDetails, IPartContentHasSupplementalData, IContextListener, Listener{

    protected final Logger logger = Logger.getLogger(DataImportView.class);

    protected Collection<T> results = new ArrayList<T>();

    protected boolean updated = false;

    protected OccurenceQuery query;

    private static ConversationHolder conversationHolder;

    private SaveImportedSpecimenAction saveImportedSpecimenAction;

    private Text textClassification;
    private Classification classification;
    /**
	 * @return the classification
	 */
	public Classification getClassification() {
		return classification;
	}

	/**
	 * @param classification the classification to set
	 */
	public void setClassification(Classification classification) {
		this.classification = classification;
	}
	private Button btnBrowseClassification;
    private Button btnClear;

    private final FormToolkit toolkit = new FormToolkit(Display.getCurrent());

    private Table table;

    /**
     * Constructs a new DataImportEditor and registers it to listen to context changes
     */
    public DataImportView() {
        CdmStore.getContextManager().addContextListener(this);
        if(CdmStore.isActive()){
            initConversation();
        }
    }

    /**
     * Create contents of the view part.
     * @param parent
     */
    @Override
    public void createPartControl(Composite parent) {
    	final Composite composite = new Composite(parent, SWT.NULL);

		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 4;
		composite.setLayout(gridLayout);
    	Label label = new Label(composite, SWT.NONE);
		label.setText("Classification");
		textClassification = new Text(composite, SWT.NONE);
		textClassification.setEnabled(false);
		textClassification.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		btnBrowseClassification = new Button(composite, SWT.NONE);
		btnBrowseClassification.setImage(ResourceManager.getPluginImage("eu.etaxonomy.taxeditor.store", "icons/open.gif"));
		btnBrowseClassification.addListener(SWT.Selection, this);
		btnClear = new Button(composite, SWT.NONE);
		btnClear.setImage(ResourceManager.getPluginImage("eu.etaxonomy.taxeditor.store", "icons/trash.gif"));
		btnClear.addListener(SWT.Selection, this);

        CheckboxTableViewer checkboxTableViewer = CheckboxTableViewer.newCheckList(parent, SWT.BORDER | SWT.FULL_SELECTION);
        table = checkboxTableViewer.getTable();
        toolkit.paintBordersFor(table);



        createActions();
        initializeToolBar();
        initializeMenu();
    }

    /**
     * Create the actions.
     */
    private void createActions() {
        saveImportedSpecimenAction = new SaveImportedSpecimenAction();
    }

    /**
     * Initialize the toolbar.
     */
    private void initializeToolBar() {
        IToolBarManager tbm = getViewSite().getActionBars().getToolBarManager();
        tbm.add(saveImportedSpecimenAction);
    }

    /**
     * Initialize the menu.
     */
    private void initializeMenu() {
        IMenuManager manager = getViewSite().getActionBars().getMenuManager();
    }

    public Table getTable() {
        return table;
    }

    /**
     * @param query the query to set
     */
    public void setQuery(OccurenceQuery query) {
        this.query = query;
    }

    /**
     * @param results the results to set
     */
    public void setResults(Collection<T> results) {
        this.results = results;
        updated=false;
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.annotatedlineeditor.AnnotatedLineEditor#dispose()
     */
    @Override
    public void dispose() {
        super.dispose();
        CdmStore.getContextManager().removeContextListener(this);
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.model.IContextListener#contextRefresh(org.eclipse.core.runtime.IProgressMonitor)
     */
    @Override
    public void contextRefresh(IProgressMonitor monitor) {
        refresh();
    }

    protected void refresh(){

        if (!updated){
            if (getTable() != null){
                getTable().removeAll();
            }
            for(T item:results){
                TableItem tableItem = new TableItem(getTable(), SWT.NONE);
                tableItem.setText(getTextForTableItem(item));
                tableItem.setData(item);
            }
            updated = true;
        }
    }

    public abstract void query();


    /* (non-Javadoc)
     * @see eu.etaxonomy.cdm.persistence.hibernate.ICdmPostDataChangeObserver#update(eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap)
     */
    @Override
    public void update(CdmDataChangeMap changeEvents) {
        //nothing
    }

    /**
     * Returns the text label of the given item.
     * @param item the item for which the label should be returns
     * @return the label of the item
     */
    protected abstract String getTextForTableItem(T item);

    /* (non-Javadoc)
     * @see org.eclipse.ui.part.WorkbenchPart#setFocus()
     */
    @Override
    public void setFocus() {
        getTable().setFocus();
        //make sure to bind again if maybe in another view the conversation was unbound
        if(getConversationHolder()!=null && !getConversationHolder().isBound()){
            getConversationHolder().bind();
        }
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.model.IContextListener#contextAboutToStop(org.eclipse.ui.IMemento, org.eclipse.core.runtime.IProgressMonitor)
     */
    @Override
    public void contextAboutToStop(IMemento memento, IProgressMonitor monitor) {
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.model.IContextListener#contextStop(org.eclipse.ui.IMemento, org.eclipse.core.runtime.IProgressMonitor)
     */
    @Override
    public void contextStop(IMemento memento, IProgressMonitor monitor) {
//        derivateSearchCompositeController.setEnabled(false);
//        derivateSearchCompositeController.reset();
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.model.IContextListener#contextStart(org.eclipse.ui.IMemento, org.eclipse.core.runtime.IProgressMonitor)
     */
    @Override
    public void contextStart(IMemento memento, IProgressMonitor monitor) {
        initConversation();
//        derivateSearchCompositeController.setEnabled(true);
    }

    private void initConversation(){
        if(conversationHolder==null){
            conversationHolder = CdmStore.createConversation();
        }
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.model.IContextListener#workbenchShutdown(org.eclipse.ui.IMemento, org.eclipse.core.runtime.IProgressMonitor)
     */
    @Override
    public void workbenchShutdown(IMemento memento, IProgressMonitor monitor) {
        if(getConversationHolder()!=null && getConversationHolder().isBound() && !getConversationHolder().isClosed()) {
            getConversationHolder().close();
        }
    }

    /**
     * @return the conversationHolder
     */
    @Override
    public ConversationHolder getConversationHolder() {
        if(CdmStore.isActive() && conversationHolder==null){
            initConversation();
        }
        return conversationHolder;
    }
    @Override
	public void handleEvent(Event event) {
	    if(event.widget==btnBrowseClassification){
	        classification = SelectionDialogFactory.getSelectionFromDialog(Classification.class, getSite().getShell(), null, null);
	        if(classification!=null){
	            textClassification.setText(classification.getTitleCache());
	        }
	    }
	    else if(event.widget==btnClear){
	        classification = null;
	        textClassification.setText("");
	    }
	}
}
